﻿using System;
using System.Collections.Generic;
using System.Text;

namespace SmarterNotify.Data
{
    public class Reminder
    {
        public enum AlertType
        {
            WebService,
            Internal,
            Plugin
        }
        // displayable attributes... DO NOT CHANGE ORDER
        public string Subject { get; set; }
        public string TimeOccurredDT
        {
            get
            {
                if (TimeOccurredUTC == DateTime.MinValue)
                    return string.Empty;

                DateTime dtLocal = TimeOccurredUTC.ToLocalTime();
                string retVal = string.Empty;

                if (TimeOccurredUTC.Date == DateTime.UtcNow.Date)
                    retVal = dtLocal.ToString("t");
                else if ((TimeOccurredUTC.Date < DateTime.UtcNow.Date && TimeOccurredUTC.Date > DateTime.UtcNow.AddDays(-7)) || 
                    (TimeOccurredUTC.Date > DateTime.UtcNow.Date && TimeOccurredUTC.Date < DateTime.UtcNow.AddDays(7)))
                    retVal = dtLocal.ToString("ddd") + ", " + dtLocal.ToString("t");
                else 
                    retVal = dtLocal.ToString("d");

                return retVal;
            }
        }
        public string SnoozeTimeDT
        {
            get
            {
                if (SnoozeTimeUTC == DateTime.MinValue)
                    return string.Empty;

                DateTime dtLocal = SnoozeTimeUTC.ToLocalTime();
                string retVal = string.Empty;

                if (SnoozeTimeUTC.Date == DateTime.UtcNow.Date)
                    retVal = dtLocal.ToString("t");
                else if ((TimeOccurredUTC.Date < DateTime.UtcNow.Date && SnoozeTimeUTC.Date > DateTime.UtcNow.AddDays(-7)) ||
                    (SnoozeTimeUTC.Date > DateTime.UtcNow.Date && SnoozeTimeUTC.Date < DateTime.UtcNow.AddDays(7)))
                    retVal = dtLocal.ToString("ddd") + ", " + dtLocal.ToString("t");
                else
                    retVal = dtLocal.ToString("d");

                return retVal;
            }
        }
        public string OwnerProfileName { get; set; }
        public string CategoryType { get; set; }

        // normal attributes
        public AlertType ReminderType { get; set; }
        public string ReminderID { get; set; } // can be a GUID or number...
        public string ProfileGUID { get; set; }
        public string PhonicSubject { get; set; }
        public string BodyText { get; set; }
        public DateTime SnoozeTimeUTC { get; set; }
        public DateTime TimeOccurredUTC { get; set; }
        
        public Reminder()
        {
            ReminderType = AlertType.WebService;
            ReminderID = Guid.NewGuid().ToString("N");
            ProfileGUID = "";
            Subject = "";
            PhonicSubject = "";
            BodyText = "";
            CategoryType = "NA";
            SnoozeTimeUTC = DateTime.MinValue;
            TimeOccurredUTC = DateTime.MinValue;
        }

        public bool IsInternalOrPlugin()
        {
            return ReminderType == Reminder.AlertType.Internal || ReminderType == Reminder.AlertType.Plugin;
        }

        public static Comparison<Reminder> TimeOccurredComparisonDESC = 
            delegate(Reminder r1, Reminder r2) { return r2.TimeOccurredUTC.CompareTo(r1.TimeOccurredUTC); };
    }// class
}// namespace