﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using SmarterNotify.HelperClasses;
using System.IO;
using System.Text.RegularExpressions;

namespace SmarterNotify.DataHandlers
{
    public class BrowserHandler
    {
        public static object BrowserLocker = new object();
        public static string LastBrowserValue = string.Empty;

        public static void MakeSureFileExists()
        {
            lock (BrowserLocker)
            {
                if (!File.Exists(AppPath.BrowserNotificationFilePath))
                    File.Create(AppPath.BrowserNotificationFilePath);
            }
        }
        public static void ClearBrowser(ref WebBrowser browser)
        {
            lock (BrowserLocker)
            {
                WriteNotification("", AppPath.BrowserNotificationFilePath);
                browser.Refresh();
            }
        }
        public static void LoadBrowserNotification(ref WebBrowser browser, string notification)
        {
            lock (BrowserLocker)
            {
                string filePath = AppPath.BrowserNotificationFilePath;
                if (WriteNotification(notification, filePath))
                {
                    if (browser.Url == null)
                        browser.Url = new Uri(filePath);

                    browser.Refresh();
                }
            }
        }
        private static bool WriteNotification(string notification, string filePath)
        {
            // returns bool = whether or not the notification was updated
            if (notification == LastBrowserValue)
                return false;

            // format the notification a little
            string formedNote = notification.Replace("\r\n", "<br />").Replace("\n\n", "<br /><br />").Replace("\n", "<br />").Replace("\r", "<br />");

            if (formedNote.Contains("<a href") || formedNote.Contains("</a>") || 
                formedNote.Contains("<a target") || formedNote.Contains("<a class") ||
                formedNote.Contains("<a style"))
            {
                List<string> formatStrings = new List<string>();
                int formatCounter = 0;
                while (formedNote.Contains("<a"))
                {
                    string formStr = formedNote.Substring(formedNote.IndexOf("<a"));
                    string formNot = formStr.Substring(0, formStr.IndexOf(">"));

                    string formatString = formNot.Replace("<a", "<a target=\"_blank\"");
                    formedNote = formedNote.Replace(formNot, "{" + formatCounter + "}");
                    formatStrings.Add(formatString);
                    formatCounter++;
                }
                formedNote = string.Format(formedNote, formatStrings.ToArray());
                formedNote = formedNote.Replace("target=\"_self\"", "target=\"_blank\"");
                formedNote = formedNote.Replace("target=\"_parent\"", "target=\"_blank\"");
                formedNote = formedNote.Replace("target=\"_top\"", "target=\"_blank\"");
            }
            else
            {
                Regex regx = new Regex("http(s)?://([\\w+?\\.\\w+])+([a-zA-Z0-9\\~\\!\\@\\#\\$\\%\\^\\&amp;\\*\\(\\)_\\-\\=\\+\\\\\\/\\?\\.\\:\\;\\'\\,]*)?", RegexOptions.IgnoreCase);
                MatchCollection mactches = regx.Matches(formedNote);
                foreach (Match match in mactches)
                {
                    formedNote = formedNote.Replace(match.Value, "<a target='blank' href='" + match.Value + "'>" + match.Value + "</a>");
                }
            }

            StringBuilder sb = new StringBuilder();
            sb.AppendLine("<html>");
            sb.AppendLine("<head>");
            sb.AppendLine("</head>");
            sb.AppendLine("<body>");
            sb.AppendLine("<div style=\"font: 9pt Arial, Verdana; color: #333333;\">");
            sb.AppendLine(formedNote);
            sb.AppendLine("</div>");
            sb.AppendLine("</body>");
            sb.AppendLine("</html>");

            string htmlVal = sb.ToString();
            
            StreamWriter sw = new StreamWriter(filePath, false);
            sw.Write(htmlVal);
            sw.Flush();
            sw.Close();

            LastBrowserValue = notification;
            return true;
        }
    }// class
}// namespace