﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Threading;
using SmarterNotify.HelperClasses;

namespace SmarterNotify.DataHandlers
{
    public class LogHandler
    {
        #region LogType enum

        public enum LogType
        {
            Error,
            Connections,
            Notifications
        }

        #endregion

        public static object LogLocker = new object();
        public static object LogWriteLocker = new object();

        public static void ClearLog(LogType type)
        {
            lock (LogWriteLocker)
            {
                string filePath = string.Empty;
                string bakPath = string.Empty;

                switch (type)
                {
                    case LogType.Error:
                        filePath = AppPath.ErrorLogFilePath;
                        bakPath = AppPath.ErrorLogBakFilePath;
                        break;
                    case LogType.Connections:
                        filePath = AppPath.ConnectionLogFilePath;
                        bakPath = AppPath.ConnectionLogBakFilePath;
                        break;
                    case LogType.Notifications:
                        filePath = AppPath.NotificationLogFilePath;
                        bakPath = AppPath.NotificationLogBakFilePath;
                        break;
                }
                if (string.IsNullOrEmpty(filePath))
                    return;

                if (!string.IsNullOrEmpty(bakPath) && File.Exists(bakPath))
                    File.Delete(bakPath);
                if (File.Exists(filePath))
                {
                    using (var sw = new StreamWriter(filePath, false))
                    {
                    }
                }
            }
        }

        public static void WriteLog(LogType type, string name)
        {
            WriteLog(type, name, null, null);
        }

        public static void WriteLog(LogType type, string name, string content)
        {
            WriteLog(type, name, null, content);
        }

        public static void WriteLog(LogType type, string name, string message, string content)
        {
            lock (LogWriteLocker)
            {
                string filePath = string.Empty;
                switch (type)
                {
                    case LogType.Error:
                        filePath = AppPath.ErrorLogFilePath;
                        break;
                    case LogType.Connections:
                        filePath = AppPath.ConnectionLogFilePath;
                        break;
                    case LogType.Notifications:
                        filePath = AppPath.NotificationLogFilePath;
                        break;
                }
                if (string.IsNullOrEmpty(filePath))
                    return;

                RotateFileIfNeeded(filePath);
                var sb = new StringBuilder();

                // Write a new log entry into the appropriate log file
                switch (type)
                {
                    case LogType.Error:
                        sb.AppendFormat("{0:d} {0:t} ({1:d} {1:t}) {2}\r\n",
                            DateTime.Now, DateTime.UtcNow, name);
                        sb.AppendLine("");
                        sb.AppendLine("Message:");
                        sb.AppendLine(message);
                        sb.AppendLine("");
                        sb.AppendLine("Stack Trace:");
                        sb.AppendLine(content);
                        break;
                    case LogType.Connections:
                        sb.AppendFormat("{0:d} {0:t} ({1:d} {1:t}) {2}\r\n",
                            DateTime.Now, DateTime.UtcNow, name);
                        break;
                    case LogType.Notifications:
                        sb.AppendFormat("{0:d} {0:t} ({1:d} {1:t}) {2}\r\n",
                            DateTime.Now, DateTime.UtcNow, name);
                        sb.AppendLine("");
                        sb.AppendLine("Message:");
                        sb.AppendLine(content);
                        break;
                }

                sb.AppendLine("");
                sb.AppendLine("-----------------------------------------------------------");

                try
                {
                    File.AppendAllText(filePath, sb.ToString());
                }
                catch (IOException)
                {
                }
            }
        }

        public static void RotateFileIfNeeded(string fileName)
        {
            try
            {
                lock (LogLocker)
                {
                    if (File.Exists(fileName) && new FileInfo(fileName).Length > 10000000)
                    {
                        try
                        {
                            File.Delete(fileName + ".bak");
                        }
                        catch
                        {
                        }
                        try
                        {
                            Thread.Sleep(20);
                            File.Move(fileName, fileName + ".bak");
                            File.Create(fileName);
                        }
                        catch
                        {
                        }
                    }
                }
                Thread.Sleep(20);
            }
            catch
            {
            }
        }
    }
}