﻿using System;
using System.Collections.Generic;
using System.Text;
using SmarterNotify.Data;
using System.IO;
using System.Xml;
using SmarterNotify.HelperClasses;

namespace SmarterNotify.DataHandlers
{
    public class SettingsHandler
    {
        public static bool HasLoadedFromFile = false;
        private static Settings globalSettings = new Settings();
        public static string SettingsKey = "";

        public static void LoadSettingsFromFile()
        {
            try
            {
                lock (SettingsKey)
                {
                    string fileLocation = AppPath.SettingsFilePath;

                    if (!File.Exists(fileLocation))
                    {
                        Settings defaultSettings = new Settings();
                        SettingsHandler.SaveSettings(defaultSettings, fileLocation);
                    }

                    XmlTextReader xmlRead = new XmlTextReader(AppPath.SettingsFilePath);
                    while (xmlRead.Read())
                    {
                        if (xmlRead.Name.ToUpperInvariant() == "SETTINGS")
                        {
                            bool parsedBool = false;
                            int parsedInt = -1;
                            xmlRead.Read();
                            while (xmlRead.Name.ToUpperInvariant() != "SETTINGS" && xmlRead.NodeType != XmlNodeType.EndElement)
                            {
                                if (xmlRead.NodeType == XmlNodeType.Element)
                                {
                                    switch (xmlRead.Name.ToUpperInvariant())
                                    {
                                        case "NOTIFICATIONSOUNDFILE":
                                            globalSettings.NotificationSoundFile = xmlRead["value"];
                                            break;
                                        case "NOTIFYPOPUPBALLOON":
                                            if (bool.TryParse(xmlRead["value"].ToLowerInvariant(), out parsedBool))
                                                globalSettings.NotifyPopupBalloon = parsedBool;
                                            break;
                                        case "NOTIFYANIMATEICON":
                                            if (bool.TryParse(xmlRead["value"].ToLowerInvariant(), out parsedBool))
                                                globalSettings.NotifyAnimateIcon = parsedBool;
                                            break;
                                        case "NOTIFYPLAYSOUND":
                                            if (bool.TryParse(xmlRead["value"].ToLowerInvariant(), out parsedBool))
                                                globalSettings.NotifyPlaySound = parsedBool;
                                            break;
                                        case "NOTIFYSHOWINTERFACE":
                                            if (bool.TryParse(xmlRead["value"].ToLowerInvariant(), out parsedBool))
                                                globalSettings.NotifyShowInterface = parsedBool;
                                            break;
                                        case "NOTASKBARMINIMIZED":
                                            if (bool.TryParse(xmlRead["value"].ToLowerInvariant(), out parsedBool))
                                                globalSettings.NoTaskbarMinimized = parsedBool;
                                            break;
                                        case "NOTIFYONLYMINIMIZED":
                                            if (bool.TryParse(xmlRead["value"].ToLowerInvariant(), out parsedBool))
                                                globalSettings.NotifyOnlyMinimized = parsedBool;
                                            break;
                                        case "ENABLEERRORLOGGING":
                                            if (bool.TryParse(xmlRead["value"].ToLowerInvariant(), out parsedBool))
                                                globalSettings.EnableErrorLogging = parsedBool;
                                            break;
                                        case "ENABLEERRORNOTIFICATIONS":
                                            if (bool.TryParse(xmlRead["value"].ToLowerInvariant(), out parsedBool))
                                                globalSettings.EnableErrorNotifications = parsedBool;
                                            break;
                                        case "ENABLECONNECTIONLOGGING":
                                            if (bool.TryParse(xmlRead["value"].ToLowerInvariant(), out parsedBool))
                                                globalSettings.EnableConnectionLogging = parsedBool;
                                            break;
                                        case "ENABLECONNECTIONNOTIFICATIONS":
                                            if (bool.TryParse(xmlRead["value"].ToLowerInvariant(), out parsedBool))
                                                globalSettings.EnableConnectionNotifications = parsedBool;
                                            break;
                                        case "ENABLENOTIFICATIONLOGGING":
                                            if (bool.TryParse(xmlRead["value"].ToLowerInvariant(), out parsedBool))
                                                globalSettings.EnableNotificationLogging = parsedBool;
                                            break;
                                        case "AUTONAVIGATENEWALERTS":
                                            if (bool.TryParse(xmlRead["value"].ToLowerInvariant(), out parsedBool))
                                                globalSettings.AutoNavigateNewAlerts = parsedBool;
                                            break;
                                        case "MINIMIZEONDISMISSALL":
                                            if (bool.TryParse(xmlRead["value"].ToLowerInvariant(), out parsedBool))
                                                globalSettings.MinimizeOnDismissAll = parsedBool;
                                            break;
                                        case "MESSAGEONFIRSTMINIMIZE":
                                            if (bool.TryParse(xmlRead["value"].ToLowerInvariant(), out parsedBool))
                                                globalSettings.MessageOnFirstMinimize = parsedBool;
                                            break;
                                        case "NOTIFYREADTOME":
                                            if (bool.TryParse(xmlRead["value"].ToLowerInvariant(), out parsedBool))
                                                globalSettings.NotifyReadToMe = parsedBool;
                                            break;
                                        case "ISFIRSTLOAD":
                                            if (bool.TryParse(xmlRead["value"].ToLowerInvariant(), out parsedBool))
                                                globalSettings.IsFirstLoad = parsedBool;
                                            break;
                                        case "SOUNDINTERVAL":
                                            if (int.TryParse(xmlRead["value"].ToLowerInvariant(), out parsedInt))
                                                globalSettings.SoundInterval = parsedInt;
                                            break;
                                    }
                                    xmlRead.Read();
                                }
                            }
                            xmlRead.Close();
                        }
                    }

                    HasLoadedFromFile = true;
                }

                #region Flat File Implementation
                // open a connection to the Connections.profile file and load all the available ones to memory
                //if (!Directory.Exists("Config"))
                //    Directory.CreateDirectory("Config");
                //if (!File.Exists("Config\\Settings.dat"))
                //    CreateDefaultSettingsFile();

                //lock (SettingsKey)
                //{
                //    StreamReader sr = new StreamReader("Config\\Settings.dat");
                //    while (!sr.EndOfStream)
                //    {
                //        string thisLine = sr.ReadLine();
                //        if (thisLine == null)
                //            break;
                //        if (!thisLine.Contains("="))
                //            continue;

                //        bool parsedBool = false;
                //        int parsedInt = -1;

                //        string[] parser = thisLine.Split('=');
                //        if (parser.Length >= 2)
                //        {
                //            switch (parser[0].ToUpperInvariant())
                //            {
                //                case "NotificationSoundFile":
                //                    globalSettings.NotificationSoundFile = parser[1];
                //                    break;
                //                case "NOTIFYPOPUPBALLOON": 
                //                    if (bool.TryParse(parser[1].ToLowerInvariant(), out parsedBool))
                //                        globalSettings.NotifyPopupBalloon = parsedBool; 
                //                    break;
                //                case "NOTIFYANIMATEICON": 
                //                    if (bool.TryParse(parser[1].ToLowerInvariant(), out parsedBool))
                //                        globalSettings.NotifyAnimateIcon = parsedBool; 
                //                    break;
                //                case "NOTIFYPLAYSOUND":
                //                    if (bool.TryParse(parser[1].ToLowerInvariant(), out parsedBool))
                //                        globalSettings.NotifyPlaySound = parsedBool; 
                //                    break;
                //                case "NOTIFYSHOWINTERFACE":
                //                    if (bool.TryParse(parser[1].ToLowerInvariant(), out parsedBool))
                //                        globalSettings.NotifyShowInterface = parsedBool;
                //                    break;
                //                case "NOTASKBARMINIMIZED":
                //                    if (bool.TryParse(parser[1].ToLowerInvariant(), out parsedBool))
                //                        globalSettings.NoTaskbarMinimized = parsedBool;
                //                    break;
                //                case "NOTIFYONLYMINIMIZED": 
                //                    if (bool.TryParse(parser[1].ToLowerInvariant(), out parsedBool))
                //                        globalSettings.NotifyOnlyMinimized = parsedBool;
                //                    break;
                //                case "ENABLEERRORLOGGING":
                //                    if (bool.TryParse(parser[1].ToLowerInvariant(), out parsedBool))
                //                        globalSettings.EnableErrorLogging = parsedBool;
                //                    break;
                //                case "ENABLECONNECTIONLOGGING":
                //                    if (bool.TryParse(parser[1].ToLowerInvariant(), out parsedBool))
                //                        globalSettings.EnableConnectionLogging = parsedBool;
                //                    break;
                //                case "ENABLENOTIFICATIONLOGGING":
                //                    if (bool.TryParse(parser[1].ToLowerInvariant(), out parsedBool))
                //                        globalSettings.EnableNotificationLogging = parsedBool;
                //                    break;
                //                case "SOUNDINTERVAL":
                //                    if (int.TryParse(parser[1].ToLowerInvariant(), out parsedInt))
                //                        globalSettings.SoundInterval = parsedInt;
                //                    break;
                //            }
                //        }
                //    }
                //    sr.Close();
                //}

                //HasLoadedFromFile = true;
                #endregion
            }
            catch (Exception ex)
            {
                string logMsg = "Error occurred while loading settings from file";
                TrackingHelper.TrackError(TrackingHelper.RecordType.Both, "Error Occurred", logMsg, logMsg, ex);
            }
        }
        private static void WriteSettingsToFile(Settings settings, string filePath)
        {
            try
            {
                string setPath = filePath == null || filePath.Length == 0 ? AppPath.SettingsFilePath : filePath;
                XmlTextWriter xmlWrite = new XmlTextWriter(setPath, null);
                xmlWrite.WriteStartDocument();
                xmlWrite.WriteStartElement("Settings");

                // Write each of the settings to the xml file
                xmlWrite.WriteStartElement("NotificationSoundFile");
                xmlWrite.WriteAttributeString("value", settings.NotificationSoundFile);
                xmlWrite.WriteEndElement();

                xmlWrite.WriteStartElement("NotifyPopupBalloon");
                xmlWrite.WriteAttributeString("value", settings.NotifyPopupBalloon.ToString());
                xmlWrite.WriteEndElement();

                xmlWrite.WriteStartElement("NotifyAnimateIcon");
                xmlWrite.WriteAttributeString("value", settings.NotifyAnimateIcon.ToString());
                xmlWrite.WriteEndElement();

                xmlWrite.WriteStartElement("NotifyPlaySound");
                xmlWrite.WriteAttributeString("value", settings.NotifyPlaySound.ToString());
                xmlWrite.WriteEndElement();

                xmlWrite.WriteStartElement("NotifyShowInterface");
                xmlWrite.WriteAttributeString("value", settings.NotifyShowInterface.ToString());
                xmlWrite.WriteEndElement();

                xmlWrite.WriteStartElement("NoTaskbarMinimized");
                xmlWrite.WriteAttributeString("value", settings.NoTaskbarMinimized.ToString());
                xmlWrite.WriteEndElement();

                xmlWrite.WriteStartElement("NotifyOnlyMinimized");
                xmlWrite.WriteAttributeString("value", settings.NotifyOnlyMinimized.ToString());
                xmlWrite.WriteEndElement();

                xmlWrite.WriteStartElement("EnableErrorLogging");
                xmlWrite.WriteAttributeString("value", settings.EnableErrorLogging.ToString());
                xmlWrite.WriteEndElement();

                xmlWrite.WriteStartElement("EnableErrorNotifications");
                xmlWrite.WriteAttributeString("value", settings.EnableErrorNotifications.ToString());
                xmlWrite.WriteEndElement();

                xmlWrite.WriteStartElement("EnableConnectionLogging");
                xmlWrite.WriteAttributeString("value", settings.EnableConnectionLogging.ToString());
                xmlWrite.WriteEndElement();

                xmlWrite.WriteStartElement("EnableConnectionNotifications");
                xmlWrite.WriteAttributeString("value", settings.EnableConnectionNotifications.ToString());
                xmlWrite.WriteEndElement();

                xmlWrite.WriteStartElement("EnableNotificationLogging");
                xmlWrite.WriteAttributeString("value", settings.EnableNotificationLogging.ToString());
                xmlWrite.WriteEndElement();

                xmlWrite.WriteStartElement("AutoNavigateNewAlerts");
                xmlWrite.WriteAttributeString("value", settings.AutoNavigateNewAlerts.ToString());
                xmlWrite.WriteEndElement();

                xmlWrite.WriteStartElement("MinimizeOnDismissAll");
                xmlWrite.WriteAttributeString("value", settings.MinimizeOnDismissAll.ToString());
                xmlWrite.WriteEndElement();

                xmlWrite.WriteStartElement("MessageOnFirstMinimize");
                xmlWrite.WriteAttributeString("value", settings.MessageOnFirstMinimize.ToString());
                xmlWrite.WriteEndElement();

                xmlWrite.WriteStartElement("NotifyReadToMe");
                xmlWrite.WriteAttributeString("value", settings.NotifyReadToMe.ToString());
                xmlWrite.WriteEndElement();

                xmlWrite.WriteStartElement("IsFirstLoad");
                xmlWrite.WriteAttributeString("value", settings.IsFirstLoad.ToString());
                xmlWrite.WriteEndElement();                

                xmlWrite.WriteStartElement("SoundInterval");
                xmlWrite.WriteAttributeString("value", settings.SoundInterval.ToString());
                xmlWrite.WriteEndElement();

                // write final </Settings>
                xmlWrite.WriteEndElement();
                xmlWrite.Flush();
                xmlWrite.Close();

                #region Flat File Implementation
                //StreamWriter sw = new StreamWriter("Config\\Settings.dat", false);

                //sw.WriteLine("NotificationSoundFile=" + settings.NotificationSoundFile.ToString());
                //sw.WriteLine("NotifyPopupBalloon=" + settings.NotifyPopupBalloon.ToString());
                //sw.WriteLine("NotifyAnimateIcon=" + settings.NotifyAnimateIcon.ToString());
                //sw.WriteLine("NotifyPlaySound=" + settings.NotifyPlaySound.ToString());
                //sw.WriteLine("NotifyShowInterface=" + settings.NotifyShowInterface.ToString());
                //sw.WriteLine("NoTaskbarMinimized=" + settings.NoTaskbarMinimized.ToString());
                //sw.WriteLine("NotifyOnlyMinimized=" + settings.NotifyOnlyMinimized.ToString());
                //sw.WriteLine("EnableErrorLogging=" + settings.EnableErrorLogging.ToString());
                //sw.WriteLine("EnableConnectionLogging=" + settings.EnableConnectionLogging.ToString());
                //sw.WriteLine("EnableNotificationLogging=" + settings.EnableNotificationLogging.ToString());
                //sw.WriteLine("SoundInterval=" + settings.SoundInterval.ToString());

                //sw.Close();
                #endregion
            }
            catch (Exception ex) 
            {
                string logMsg = "Error occurred while writing settings to file";
                TrackingHelper.TrackError(TrackingHelper.RecordType.Both, "Error Occurred", logMsg, logMsg, ex);
            }
        }

        public static Settings GlobalSettings
        {
            get
            {
                lock (SettingsKey)
                {
                    if (!HasLoadedFromFile)
                        LoadSettingsFromFile();

                    return globalSettings;
                }
            }
            set
            {
                lock (SettingsKey)
                {
                    SaveSettings(value);
                }
            }
        }
        public static void SaveSettings()
        {
            lock (SettingsKey)
            {
                WriteSettingsToFile(globalSettings, null);
            }
        }
        public static void SaveSettings(Settings settings)
        {
            lock (SettingsKey)
            {
                globalSettings = settings;
                WriteSettingsToFile(settings, null);
            }
        }
        public static void SaveSettings(Settings settings, string filePath)
        {
            lock (SettingsKey)
            {
                globalSettings = settings;
                WriteSettingsToFile(settings, filePath);
            }
        }
    }
}