﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using SmarterNotify.Data;
using SmarterNotify.Data.Voice;
using SmarterNotify.HelperClasses;
using System.IO;
using System.Xml;
using System.Text.RegularExpressions;

namespace SmarterNotify.DataHandlers
{
    public class VoiceDictionaryHandler
    {
        public static Dictionary<string, VoiceEntry> VoiceDictionary = new Dictionary<string, VoiceEntry>();
        public static object VoiceLocker = new object();

        public static void LoadVoiceEntriesFromFile()
        {
            try
            {
                // open a connection to the Connections file and load all the available ones to memory
                LoadVoiceEntriesFromFile(AppPath.VoiceDictionaryFilePath);
            }
            catch (Exception ex)
            {
                string logMsg = "Error occurred while loading voice entries from file";
                TrackingHelper.TrackError(TrackingHelper.RecordType.Both, "Error Occurred", logMsg, logMsg, ex);
            }
        }
        public static void LoadVoiceEntriesFromFile(string fileLocation)
        {
            lock (VoiceLocker)
            {
                foreach (VoiceEntry voice in GetVoiceEntriesFromFile(fileLocation))
                    VoiceDictionary[voice.Key] = voice;
            }
        }
        public static List<VoiceEntry> GetVoiceEntriesFromFile(string fileLocation)
        {
            List<VoiceEntry> retVal = new List<VoiceEntry>();
            FileInfo info = new FileInfo(fileLocation);
            if (info.Extension.ToUpperInvariant().Contains("XML"))
            {
                try
                {
                    if (!File.Exists(fileLocation))
                    {
                        XmlTextWriter xmlWrite = new XmlTextWriter(fileLocation, null);
                        xmlWrite.WriteStartDocument();
                        xmlWrite.WriteStartElement("VoiceEntries");
                        WriteDefaultEntries(ref xmlWrite);
                        xmlWrite.WriteEndElement();
                        xmlWrite.Flush();
                        xmlWrite.Close();
                    }

                    XmlTextReader xmlRead = new XmlTextReader(fileLocation);

                    while (xmlRead.Read())
                    {
                        switch (xmlRead.NodeType)
                        {
                            case XmlNodeType.Element:
                                if (xmlRead.Name == "VoiceEntry")
                                {
                                    VoiceEntry voiceEntry = InterpretVoiceEntry(ref xmlRead);
                                    if (voiceEntry != null)
                                        retVal.Add(voiceEntry);
                                }
                                break;
                        }
                    }

                    xmlRead.Close();
                }
                catch (Exception ex)
                {
                    string logMsg = "Error occurred while getting voice entries from file";
                    TrackingHelper.TrackError(TrackingHelper.RecordType.Both, "Error Occurred", logMsg, logMsg, ex);
                }
            }
            return retVal;
        }
        public static VoiceEntry InterpretVoiceEntry(ref XmlTextReader xmlRead)
        {
            VoiceEntry loadedProfile = new VoiceEntry();
            while (xmlRead.MoveToNextAttribute())
            {
                switch (xmlRead.Name)
                {
                    case "entryType":
                        switch (xmlRead.Value.ToUpperInvariant().Trim())
                        {
                            case "REPLACETEXT": loadedProfile.EntryType = VoiceEntry.VoiceEntryType.ReplaceText; break;
                            case "REPLACEREGEX": loadedProfile.EntryType = VoiceEntry.VoiceEntryType.ReplaceRegex; break;
                        }
                        break;
                    case "key": loadedProfile.Key = xmlRead.Value; break;
                    case "val": loadedProfile.Val = xmlRead.Value; break;
                }
            }
            return loadedProfile;
        }
        public static void WriteDefaultEntries(ref XmlTextWriter xmlWrite)
        {
            WriteEntry(ref xmlWrite, new VoiceEntry(VoiceEntry.VoiceEntryType.ReplaceText, "KB", "K B"));
            WriteEntry(ref xmlWrite, new VoiceEntry(VoiceEntry.VoiceEntryType.ReplaceText, "MB", "M B"));
            WriteEntry(ref xmlWrite, new VoiceEntry(VoiceEntry.VoiceEntryType.ReplaceText, "GB", "G B"));
            WriteEntry(ref xmlWrite, new VoiceEntry(VoiceEntry.VoiceEntryType.ReplaceText, "TB", "T B"));
            WriteEntry(ref xmlWrite, new VoiceEntry(VoiceEntry.VoiceEntryType.ReplaceText, "Mon,", "Monday,"));
            WriteEntry(ref xmlWrite, new VoiceEntry(VoiceEntry.VoiceEntryType.ReplaceText, "Tue,", "Tuesday,"));
            WriteEntry(ref xmlWrite, new VoiceEntry(VoiceEntry.VoiceEntryType.ReplaceText, "Wed,", "Wednesday,"));
            WriteEntry(ref xmlWrite, new VoiceEntry(VoiceEntry.VoiceEntryType.ReplaceText, "Thu,", "Thursday,"));
            WriteEntry(ref xmlWrite, new VoiceEntry(VoiceEntry.VoiceEntryType.ReplaceText, "Fri,", "Friday,"));
            WriteEntry(ref xmlWrite, new VoiceEntry(VoiceEntry.VoiceEntryType.ReplaceText, "Sat,", "Saturday,"));
            WriteEntry(ref xmlWrite, new VoiceEntry(VoiceEntry.VoiceEntryType.ReplaceText, "Sun,", "Sunday,"));
            WriteEntry(ref xmlWrite, new VoiceEntry(VoiceEntry.VoiceEntryType.ReplaceText, "4", "forr"));            
            WriteEntry(ref xmlWrite, 
                new VoiceEntry(VoiceEntry.VoiceEntryType.ReplaceRegex, 
                @"\[?(?<ticketnumber>[\da-fA-F]{3}-[\da-fA-F]{8}-[\da-fA-F]{4})\]?", 
                ""));
        }
        public static void WriteEntry(ref XmlTextWriter xmlWrite, VoiceEntry entry)
        {
            xmlWrite.WriteStartElement("VoiceEntry");
            xmlWrite.WriteAttributeString("entryType", entry.EntryType.ToString());
            xmlWrite.WriteAttributeString("key", entry.Key);
            xmlWrite.WriteAttributeString("val", entry.Val);
            xmlWrite.WriteEndElement();
        }
        public static void WriteVoiceEntriesToFile()
        {
            // open a connection to the Connections.profile file and load all the available ones to memory
            try
            {
                WriteVoiceEntriesToFile(AppPath.VoiceDictionaryFilePath, new List<VoiceEntry>(VoiceDictionary.Values));
            }
            catch (Exception ex)
            {
                string logMsg = "Error occurred while writing voice entries to file";
                TrackingHelper.TrackError(TrackingHelper.RecordType.Both, "Error Occurred", logMsg, logMsg, ex);
            }
        }
        public static void WriteVoiceEntriesToFile(string fileLocation)
        {
            WriteVoiceEntriesToFile(fileLocation, new List<VoiceEntry>(VoiceDictionary.Values));
        }
        public static void WriteVoiceEntriesToFile(string fileLocation, List<VoiceEntry> voiceEntries)
        {
            lock (VoiceLocker)
            {
                XmlTextWriter xmlWrite = new XmlTextWriter(fileLocation, null);
                xmlWrite.WriteStartDocument();
                xmlWrite.WriteStartElement("VoiceEntries");
                foreach (VoiceEntry entry in voiceEntries)
                    WriteEntry(ref xmlWrite, entry);
                xmlWrite.WriteEndElement();
                xmlWrite.Flush();
                xmlWrite.Close();
            }
        }
        public static string PhonicConvert(string originalText)
        {
            lock (VoiceLocker)
            {
                string retVal = originalText;                
                foreach (KeyValuePair<string, VoiceEntry> voice in VoiceDictionary)
                {
                    switch (voice.Value.EntryType)
                    {
                        case VoiceEntry.VoiceEntryType.ReplaceRegex:
                            retVal = Regex.Replace(retVal, voice.Value.Key, voice.Value.Val);
                            break;
                        case VoiceEntry.VoiceEntryType.ReplaceText:
                            List<string> usedWords = new List<string>(retVal.Split(' '));
                            for (int i = 0; i < usedWords.Count; i++)
                                if (usedWords[i] == voice.Value.Key)
                                {
                                    usedWords[i] = voice.Value.Val;
                                    break;
                                }
                            retVal = string.Join(" ", usedWords.ToArray());
                            break;
                    }
                }
                return retVal;
            }
        }
    }
}