﻿using System;
using System.Collections.Generic;
using System.Text;
using SmarterNotify.Data;
using SmarterNotify.DataHandlers;
using SmarterNotify.STWebSvc;

namespace SmarterNotify.HelperClasses
{
    /// <summary>
    /// This helper class was designed to function ONLY with SmarterTrack servers. Refer to the appropriate helper class
    /// if you need to access a different product's web services for notifications.
    /// </summary>
    public class STWebServices
    {
        public static object ServiceLocker = new object();

        public static bool IsServiceAvailable(ConnectionProfile profile)
        {
            lock (ServiceLocker)
            {
                bool retVal = false;

                try
                {
                    STWebSvc.svcNotifications webService = GetWebService(profile.ConnectionURL);
                    if (webService != null)
                    {
                        STWebSvc.IsAvailableResult result = webService.PingService(profile.AuthUsername, profile.AuthPassword, profile.DataUsername);
                        retVal = result.IsAvailable;
                    }
                }
                catch { }

                return retVal;
            }
        }
        public static List<Reminder> GetAllReminders(ConnectionProfile profile, bool includeSnoozedItems)
        {
            lock (ServiceLocker)
            {
                STWebSvc.svcNotifications webService = GetWebService(profile.ConnectionURL);
                List<Reminder> retVal = new List<Reminder>();
                if (webService != null)
                {
                    STWebSvc.RemindersResult results = webService.GetAllReminders(profile.AuthUsername, profile.AuthPassword, profile.DataUsername, null);
                    if (results.Result == true)
                        foreach (ReminderValue result in results.Reminders)
                        {
                            Reminder usedReminder = GetTranslatedReminder(result);
                            bool isSnoozing = usedReminder.SnoozeTimeUTC > DateTime.UtcNow;
                            if (!includeSnoozedItems && isSnoozing)
                                continue;
                            else
                            {
                                usedReminder.OwnerProfileName = profile.ProfileName;
                                usedReminder.ProfileGUID = profile.ProfileGUID;
                                retVal.Add(usedReminder);
                            }
                        }
                }
                return retVal;
            }
        }

        public static Reminder GetReminder(ConnectionProfile profile, string reminderID)
        {
            lock (ServiceLocker)
            {
                STWebSvc.svcNotifications webService = GetWebService(profile.ConnectionURL);
                STWebSvc.ReminderResult result = webService.GetReminder(profile.AuthUsername, profile.AuthPassword, profile.DataUsername, reminderID, null);
                return result.Result ? GetTranslatedReminder(result.Reminder) : null;
            }
        }

        public static int GetReminderCount(ConnectionProfile profile)
        {
            lock (ServiceLocker)
            {
                STWebSvc.svcNotifications webService = GetWebService(profile.ConnectionURL);
                STWebSvc.IntRequestResult result = webService.GetReminderCount(profile.AuthUsername, profile.AuthPassword, profile.DataUsername, null);
                return result.Result ? result.RequestResult : 0;
            }
        }

        public static int GetNewReminderCount(ConnectionProfile profile, DateTime lastCheckDateUTC)
        {
            lock (ServiceLocker)
            {
                STWebSvc.svcNotifications webService = GetWebService(profile.ConnectionURL);
                string lastCheckDateString = lastCheckDateUTC.ToString();
                STWebSvc.IntRequestResult result = webService.GetNewReminderCount(profile.AuthUsername, profile.AuthPassword, profile.DataUsername, lastCheckDateString, null);
                return result.Result ? result.RequestResult : 0;
            }
        }

        public static void DismissAllReminders(ConnectionProfile profile)
        {
            lock (ServiceLocker)
            {
                STWebSvc.svcNotifications webService = GetWebService(profile.ConnectionURL);
                STWebSvc.GenericResult result = webService.DismissAllReminders(profile.AuthUsername, profile.AuthPassword, profile.DataUsername, null);
            }
        }

        public static void DissmissReminder(ConnectionProfile profile, string reminderID)
        {
            lock (ServiceLocker)
            {
                STWebSvc.svcNotifications webService = GetWebService(profile.ConnectionURL);
                STWebSvc.GenericResult result = webService.DismissReminder(profile.AuthUsername, profile.AuthPassword, profile.DataUsername, reminderID, null);
            }
        }

        public static void SnoozeReminder(ConnectionProfile profile, string reminderID, int snoozeSeconds)
        {
            lock (ServiceLocker)
            {
                STWebSvc.svcNotifications webService = GetWebService(profile.ConnectionURL);
                STWebSvc.GenericResult result = webService.SnoozeNotification(profile.AuthUsername, profile.AuthPassword, profile.DataUsername, reminderID, snoozeSeconds, null);
            }
        }

        #region Helper Classes
        private static STWebSvc.svcNotifications GetWebService(string connectionURL)
        {
            if (!connectionURL.ToUpperInvariant().StartsWith("HTTP://") && !connectionURL.ToUpperInvariant().StartsWith("HTTPS://"))
                return null;

            STWebSvc.svcNotifications retVal = new SmarterNotify.STWebSvc.svcNotifications();
            retVal.Url = connectionURL;
            
            return retVal;
        }
        private static Reminder GetTranslatedReminder(ReminderValue result)
        {
            Reminder reminder = new Reminder();
            if (result != null && result.ReminderAttributes != null)
            {
                DateTime parsedDT = DateTime.MinValue;
                foreach (ReminderValuePair pair in result.ReminderAttributes)
                {
                    switch (pair.Key)
                    {
                        case ReminderAttributes.ReminderID: reminder.ReminderID = pair.Val; break;
                        case ReminderAttributes.Subject: reminder.Subject = pair.Val; break;
                        case ReminderAttributes.PhonicSubject: reminder.PhonicSubject = pair.Val; break;
                        case ReminderAttributes.CategoryType: reminder.CategoryType = pair.Val; break;
                        case ReminderAttributes.BodyText: reminder.BodyText = pair.Val; break;
                        case ReminderAttributes.SnoozeTimeUTC:
                            if (DateTime.TryParse(pair.Val, out parsedDT) && parsedDT > DateTime.MinValue)
                                reminder.SnoozeTimeUTC = parsedDT;
                            break;
                        case ReminderAttributes.TimeOccurredUTC:
                            if (DateTime.TryParse(pair.Val, out parsedDT) && parsedDT > DateTime.MinValue)
                                reminder.TimeOccurredUTC = parsedDT;
                            break;
                    }
                }
            }
            return reminder;
            #region Unused
            //return reminder;

            //Reminder reminder = new Reminder();
            //string[] attr = result.Split('|');
            //foreach (string atb in attr)
            //{
            //    int parsedInt = -1;
            //    DateTime parsedDT = DateTime.MinValue;

            //    string[] parser = atb.Split('=');
            //    if (parser.Length >= 2)
            //    {
            //        switch (parser[0].ToUpperInvariant())
            //        {
            //            case "REMINDERID":
            //                if (int.TryParse(parser[1], out parsedInt) && parsedInt > -1)
            //                    reminder.ReminderID = parsedInt;
            //                break;
            //            case "USERID":
            //                if (int.TryParse(parser[1], out parsedInt) && parsedInt > -1)
            //                    reminder.UserID = parsedInt;
            //                break;
            //            case "SUBJECT": reminder.Subject = parser[1]; break;
            //            case "BODYTEXT": reminder.BodyText = parser[1]; break;
            //            case "SNOOZETIMEUTC":
            //                if (DateTime.TryParse(parser[1], out parsedDT) && parsedDT > DateTime.MinValue)
            //                    reminder.SnoozeTimeUTC = parsedDT;
            //                break;
            //            case "TIMEOCCURREDUTC":
            //                if (DateTime.TryParse(parser[1], out parsedDT) && parsedDT > DateTime.MinValue)
            //                    reminder.TimeOccurredUTC = parsedDT;
            //                break;
            //        }
            //    }
            //}
            #endregion
        }
        #endregion
    }
}