﻿using System;
using System.Collections.Generic;
using System.Text;
using SmarterNotify.DataHandlers;
using SmarterNotify.Data.Events;

namespace SmarterNotify.HelperClasses
{
    public class TrackingHelper
    {
        public enum RecordType
        {
            Both,
            Log,
            Notify
        }

        public static void TrackConnection(RecordType recordType, string evtName, string evtMessage, string logMessage)
        {
            bool writeLog = recordType == RecordType.Log || recordType == RecordType.Both;
            bool writeNotification = recordType == RecordType.Notify || recordType == RecordType.Both;

            if (writeNotification && SettingsHandler.GlobalSettings.EnableConnectionNotifications)
            {
                ReminderEvent evt = new ReminderEvent();
                evt.Name = evtName;
                evt.Message = evtMessage;
                evt.FireEvent();
            }
            if (writeLog && SettingsHandler.GlobalSettings.EnableConnectionLogging)
                LogHandler.WriteLog(LogHandler.LogType.Connections, logMessage);
        }
        public static void TrackError(RecordType recordType, string evtName, string evtMessage, string logMessage, Exception ex)
        {
            bool writeLog = recordType == RecordType.Log || recordType == RecordType.Both;
            bool writeNotification = recordType == RecordType.Notify || recordType == RecordType.Both;

            if (writeNotification && SettingsHandler.GlobalSettings.EnableErrorNotifications)
            {
                ReminderEvent evt = new ReminderEvent();
                evt.Name = evtName;
                evt.Message = evtMessage + "\r\n\r\nMessage:\r\n\r\n" + ex.Message + 
                    "\r\n\r\nStack Trace:\r\n\r\n" + ex.StackTrace;
                evt.FireEvent();
            }
            if (writeLog && SettingsHandler.GlobalSettings.EnableErrorLogging)
                LogHandler.WriteLog(LogHandler.LogType.Error, logMessage, ex.Message, ex.StackTrace);
        }
        public static void TrackNotification(string subject, string bodyText)
        {
            if (SettingsHandler.GlobalSettings.EnableNotificationLogging)
                LogHandler.WriteLog(LogHandler.LogType.Notifications, subject, bodyText);
        }
    }
}