﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Drawing;
using System.IO;
using SmarterNotify.Properties;
using SmarterNotify.Data;
using SmarterNotify.DataHandlers;
using SmarterNotify.Threads;

namespace SmarterNotify.HelperClasses
{
    public class TreeViewHelper
    {
        public const string TreeAllNotifications = "tvAllNotifications";
        public const string TreeSmarterNotify = "tvSmarterNotify";
        public const string TreeSmarterMail = "tvSmarterMail";
        public const string TreeSmarterStats = "tvSmarterStats";
        public const string TreeSmarterTrack = "tvSmarterTrack";
        public const string TreeExternalProvider = "tvExternalProvider";
        public const string TreePluginProvider = "tvPluginProvider";

        public const int StatusActiveAllIndex = 0;
        public const int StatusActiveSomeIndex = 1;
        public const int StatusInactiveIndex = 2;
        public const int OpenStatusIndex = 3;
        public const int ClosedStatusIndex = 4;
        public const int StatusPausedIndex = 5;
        public const int FolderIndex = 6;
        public const int WaitingStatusIndex = 7;

        public static void GenerateTreeNodes(ref TreeView tvConnections)
        {
            tvConnections.Nodes.Clear();

            bool setImages = CheckImages();
            if (setImages)
            {
                tvConnections.ImageList = new ImageList();
                tvConnections.ImageList.Images.Add(Image.FromFile("Images\\StatusActiveAll.gif"));
                tvConnections.ImageList.Images.Add(Image.FromFile("Images\\StatusActiveSome.gif"));
                tvConnections.ImageList.Images.Add(Image.FromFile("Images\\StatusInactive.gif"));
                tvConnections.ImageList.Images.Add(Image.FromFile("Images\\OpenStatus.gif"));
                tvConnections.ImageList.Images.Add(Image.FromFile("Images\\ClosedStatus.gif"));
                tvConnections.ImageList.Images.Add(Image.FromFile("Images\\StatusPaused.gif"));
                tvConnections.ImageList.Images.Add(Image.FromFile("Images\\folder.gif"));
                tvConnections.ImageList.Images.Add(Image.FromFile("Images\\WaitingStatus.gif"));
            }

            TreeNode rootNode = AddTreeRoot(ref tvConnections, TreeAllNotifications, "All Notifications", 6, setImages);
            TreeNode snNode = null;
            if (SettingsHandler.GlobalSettings.ShouldShowSmarterNotify())
                snNode = AddTreeRoot(ref tvConnections, TreeSmarterNotify, "SmarterNotify", 6, setImages);
            TreeNode smNode = GetTreeNode(TreeSmarterMail, "SmarterMail", 6, setImages);
            TreeNode ssNode = GetTreeNode(TreeSmarterStats, "SmarterStats", 6, setImages);
            TreeNode stNode = GetTreeNode(TreeSmarterTrack, "SmarterTrack", 6, setImages);  
            TreeNode extNode = GetTreeNode(TreeExternalProvider, "Web Service Providers", 6, setImages);
            TreeNode plgNode = GetTreeNode(TreePluginProvider, "Plugin Providers", 6, setImages);

            foreach (ConnectionProfile usedProfile in ConnectionProfileHandler.GetAllProfiles())
            {
                int imageIndex = usedProfile.ConnectionStatus == ConnectionProfile.ConnectionStatusType.Started ? 3 : 7;
                switch (usedProfile.ConnectionType)
                {
                    case ConnectionProfile.ConnectionProfileType.SmarterMail:
                        AddTreeNode(ref smNode, usedProfile.ProfileGUID, usedProfile.ProfileName, imageIndex, true);
                        break;
                    case ConnectionProfile.ConnectionProfileType.SmarterStats:
                        AddTreeNode(ref ssNode, usedProfile.ProfileGUID, usedProfile.ProfileName, imageIndex, true);
                        break;
                    case ConnectionProfile.ConnectionProfileType.SmarterTrack:
                        AddTreeNode(ref stNode, usedProfile.ProfileGUID, usedProfile.ProfileName, imageIndex, true);
                        break;
                    case ConnectionProfile.ConnectionProfileType.ExternalProvider:
                        AddTreeNode(ref extNode, usedProfile.ProfileGUID, usedProfile.ProfileName, imageIndex, true);
                        break;
                    case ConnectionProfile.ConnectionProfileType.PluginProvider:
                        if (PluginHandler.IsPluginAvailable(usedProfile.ProfileGUID))
                            AddTreeNode(ref plgNode, usedProfile.ProfileGUID, usedProfile.ProfileName, imageIndex, true);
                        break;
                }
            }

            //if (smNode.Nodes.Count > 0)
            //{
            tvConnections.Nodes.Add(smNode);
            smNode.ExpandAll();
            //}
            //if (stNode.Nodes.Count > 0)
            //{
            tvConnections.Nodes.Add(stNode);
            stNode.ExpandAll();
            //}
            //if (ssNode.Nodes.Count > 0)
            //{
                //tvConnections.Nodes.Add(ssNode);
                //ssNode.ExpandAll();
            //}
            //if (extNode.Nodes.Count > 0)
            //{
            tvConnections.Nodes.Add(extNode);
            extNode.ExpandAll();
            //}
            //if (plgNode.Nodes.Count > 0)
            //{
            tvConnections.Nodes.Add(plgNode);
            plgNode.ExpandAll();
            //}

            UpdateTreeNodes(ref tvConnections);
            rootNode.ExpandAll();

            // pick the node that was selected previously
            string currentProfileName = UpdateThread.CurrentProfileName;
            bool usesTopNode = false;
            switch (currentProfileName.ToString())
            {
                case TreeSmarterNotify: if (snNode != null) { tvConnections.SelectedNode = snNode; usesTopNode = true; } break;
                case TreeAllNotifications: tvConnections.SelectedNode = rootNode; usesTopNode = true; break;
            }
            if (!usesTopNode)
            {
                if (string.IsNullOrEmpty(currentProfileName) && tvConnections.Nodes.Count > 0)
                    tvConnections.SelectedNode = tvConnections.Nodes[0];
                else
                {
                    bool hasFoundNode = false;
                    foreach (TreeNode node in tvConnections.Nodes)
                    {
                        foreach (TreeNode childNode in node.Nodes)
                        {
                            if (childNode.Name == currentProfileName)
                            {
                                tvConnections.SelectedNode = childNode;
                                hasFoundNode = true;
                                break;
                            }
                        }
                        if (hasFoundNode)
                            break;
                    }
                }
            }
            tvConnections.Focus();
        }
        
        public static void UpdateTreeNodes(ref TreeView tvConnections)
        {
            if (tvConnections.Nodes.Count == 0)
                return;
            TreeNode rootNode = null;
            TreeNode snNode = null;
            TreeNode smNode = null;
            TreeNode stNode = null;
            TreeNode ssNode = null;
            TreeNode extNode = null;
            TreeNode plgNode = null;

            foreach (TreeNode node in tvConnections.Nodes)
            {
                switch (node.Name)
                {
                    case TreeAllNotifications: rootNode = node; break;
                    case TreeSmarterNotify: snNode = node; break;
                    case TreeSmarterMail: smNode = node; break;
                    case TreeSmarterStats: ssNode = node; break;
                    case TreeSmarterTrack: stNode = node; break;
                    case TreeExternalProvider: extNode = node; break;
                    case TreePluginProvider: plgNode = node; break;
                }
            }

            int totalReminders = 0;
            UpdateProductNodes(smNode, ref totalReminders);
            UpdateProductNodes(stNode, ref totalReminders);
            UpdateProductNodes(ssNode, ref totalReminders);
            UpdateProductNodes(extNode, ref totalReminders);
            UpdateProductNodes(snNode, ref totalReminders);
            UpdateProductNodes(plgNode, ref totalReminders);

            AddEndCount(rootNode, "All Notifications", totalReminders);
        }

        private static void UpdateProductNodes(TreeNode updateNode, ref int totalReminders)
        {
            if (updateNode != null)
            {
                int productReminders = 0;
                if (updateNode.Name == TreeSmarterNotify)
                {
                    // for internal reminders system
                    int internalRemindersCount = 0;
                    UpdateThread.ProfileReminderCounts.TryGetValue(UpdateThread.INTERNALREMINDERSKEY, out internalRemindersCount);
                    AddEndCount(updateNode, "SmarterNotify", internalRemindersCount);
                    productReminders += internalRemindersCount;
                }
                else
                {
                    // for all other profiles and products...
                    foreach (TreeNode node in updateNode.Nodes)
                    {
                        ConnectionProfile profile = ConnectionProfileHandler.GetProfile(node.Name);

                        int profileReminders = 0;
                        if (UpdateThread.ProfileReminderCounts.TryGetValue(node.Name, out profileReminders) && profileReminders > -1)
                            productReminders += profileReminders;
                            
                        if (profile != null)
                        {
                            AddEndCount(node, profile.ProfileName, profileReminders);
                            UpdateStatusImage(node, profile.ConnectionStatus);
                        }
                    }

                    string productString = string.Empty;
                    switch (updateNode.Name)
                    {
                        case TreeAllNotifications: productString = "All Notifications"; break;
                        case TreeSmarterMail: productString = "SmarterMail"; break;
                        case TreeSmarterTrack: productString = "SmarterTrack"; break;
                        case TreeSmarterStats: productString = "SmarterStats"; break;
                        case TreeExternalProvider: productString = "Web Service Providers"; break;
                        case TreePluginProvider: productString = "Plugin Providers"; break;
                    }

                    AddEndCount(updateNode, productString, productReminders);
                }
                totalReminders += productReminders;
            }
        }

        private static TreeNode AddTreeNode(ref TreeNode parentNode, string name, string text, int imageIndex, bool setImages)
        {
            TreeNode treeNode = new TreeNode();
            treeNode.Text = text;
            if (setImages)
            {
                treeNode.ImageIndex = imageIndex;
                treeNode.SelectedImageIndex = imageIndex;
            }
            treeNode.Name = name;
            parentNode.Nodes.Add(treeNode);
            return treeNode;
        }
        private static TreeNode AddTreeRoot(ref TreeView treeView, string name, string text, int imageIndex, bool setImages)
        {
            TreeNode treeNode = new TreeNode();
            treeNode.Text = text;
            if (setImages)
            {
                treeNode.ImageIndex = imageIndex;
                treeNode.SelectedImageIndex = imageIndex;
            }
            treeNode.Name = name;
            treeView.Nodes.Add(treeNode);
            return treeNode;
        }
        private static TreeNode GetTreeNode(string name, string text, int imageIndex, bool setImages)
        {
            TreeNode treeNode = new TreeNode();
            treeNode.Text = text;
            if (setImages)
            {
                treeNode.ImageIndex = imageIndex;
                treeNode.SelectedImageIndex = imageIndex;
            }
            treeNode.Name = name;
            return treeNode;
        }
        private static void AddEndCount(TreeNode node, string name, int count)
        {
            if (node != null)
            {
                if (count > 0)
                    node.Text = name + " (" + count + ")";
                else
                    node.Text = name;
            }
        }
        private static void UpdateStatusImage(TreeNode node, ConnectionProfile.ConnectionStatusType connectionStatus)
        {
            if (connectionStatus == ConnectionProfile.ConnectionStatusType.Started)
            {
                node.ImageIndex = OpenStatusIndex;
                node.SelectedImageIndex = OpenStatusIndex;
            }
            else if (connectionStatus == ConnectionProfile.ConnectionStatusType.Paused)
            {
                node.ImageIndex = WaitingStatusIndex;
                node.SelectedImageIndex = WaitingStatusIndex;
            }
            else
            {
                node.ImageIndex = ClosedStatusIndex;
                node.SelectedImageIndex = ClosedStatusIndex;
            }
        }
        private static void UpdateStatusImage(TreeNode node, int activeCount, int? activeSomeCount)
        {
            if (activeCount == node.Nodes.Count)
            {
                node.ImageIndex = StatusActiveAllIndex;
                node.SelectedImageIndex = StatusActiveAllIndex;
            }
            else if (activeCount > 0 || (activeSomeCount != null && activeSomeCount.Value > 0))
            {
                node.ImageIndex = StatusActiveSomeIndex;
                node.SelectedImageIndex = StatusActiveSomeIndex;
            }
            else
            {
                node.ImageIndex = StatusInactiveIndex;
                node.SelectedImageIndex = StatusInactiveIndex;
            }
        }
        //private static void UpdateStatusImages(TreeNode node, bool toggle)
        //{
        //    if (node.ImageIndex == StatusActiveAllIndex && toggle)
        //        return;
        //    if (node.ImageIndex == StatusInactiveIndex && !toggle)
        //        return;
            
        //    if (toggle)
        //        node.ImageIndex = StatusActiveAllIndex;
        //    else
        //        node.ImageIndex = StatusInactiveIndex;

        //    TreeNode parentNode = new TreeNode();
        //    while (parentNode.Parent != null)
        //    {
        //        parentNode = parentNode.Parent;

        //        int activeCount = 0;
        //        foreach (TreeNode usedNode in parentNode.Nodes)
        //            if (usedNode.ImageIndex == StatusActiveAllIndex)
        //                activeCount++;

        //        if (activeCount == parentNode.Nodes.Count)
        //            parentNode.ImageIndex = StatusActiveAllIndex;
        //        else if (activeCount > 0)
        //            parentNode.ImageIndex = StatusActiveSomeIndex;
        //        else
        //            parentNode.ImageIndex = StatusInactiveIndex;
        //    }
        //}
        private static bool CheckImages()
        {
            return File.Exists("Images\\StatusActiveAll.gif") &&
                File.Exists("Images\\StatusActiveSome.gif") &&
                File.Exists("Images\\StatusInactive.gif") &&
                File.Exists("Images\\OpenStatus.gif") &&
                File.Exists("Images\\ClosedStatus.gif") &&
                File.Exists("Images\\StatusPaused.gif") &&
                File.Exists("Images\\folder.gif");
        }
    }
}