﻿using System;
using System.Collections.Generic;
using System.Text;
using SmarterNotify.DataHandlers;
using System.IO;
using SmarterNotify.Data;
using System.Text.RegularExpressions;
using System.Reflection;

namespace SmarterNotify.HelperClasses
{
    public class ViewManager
    {
        public enum HelpCriteria
        {
            HomePage,
            MainInterface,
            WelcomeWindow,
            ConnectionProfile,
            ConnectionProfileConflict,
            Import,
            Export,
            ImportConflict,
            GettingStarted,
            Settings
        }

        public static string HelpLink = "";
        public static DateTime LastCheckDateUTC = DateTime.UtcNow;
        public static void UpdateLastCheckDate()
        {
            LastCheckDateUTC = DateTime.UtcNow;
            LogHandler.WriteLog(LogHandler.LogType.Notifications, "Updated LastCheckDateUTC", "Updated the last DateTime SmarterNotify checked new notification");
        }
        public static List<Reminder> GetNewNotifications(List<Reminder> allReminders, List<Reminder> intReminders)
        {
            List<Reminder> retVal = new List<Reminder>();
            foreach (Reminder reminder in allReminders)
            {
                bool isSnoozedExpired = reminder.SnoozeTimeUTC > DateTime.MinValue && reminder.SnoozeTimeUTC <= DateTime.UtcNow;
                if (isSnoozedExpired || reminder.TimeOccurredUTC > LastCheckDateUTC)
                    retVal.Add(reminder);

                if (isSnoozedExpired)
                {
                    // reset the snoozed time
                    ConnectionProfile profile = ConnectionProfileHandler.GetProfile(reminder.ProfileGUID);
                    profile.SnoozeReminder(reminder.ReminderID, -1);
                }
            }
            foreach (Reminder reminder in intReminders)
            {
                bool isSnoozedExpired = reminder.SnoozeTimeUTC > DateTime.MinValue && reminder.SnoozeTimeUTC <= DateTime.UtcNow;
                if (isSnoozedExpired || reminder.TimeOccurredUTC > LastCheckDateUTC)
                    retVal.Add(reminder);

                if (isSnoozedExpired)
                {
                    // reset the snoozed time
                    ConnectionProfile profile = ConnectionProfileHandler.GetProfile(reminder.ProfileGUID);
                    profile.SnoozeReminder(reminder.ReminderID, -1);
                }
            }
            return retVal;
        }
        public static void OpenHelp(HelpCriteria helpType)
        {
            string helpLink = "http://help.smartertools.com/SmarterNotify/v1/Default.aspx?p=A&v=" +
                        Assembly.GetExecutingAssembly().GetName(false).Version.ToString(3) +
                        "&lang=en-US";

            switch (helpType)
            {
                case HelpCriteria.HomePage: helpLink += "&page=smarternotify/home"; break;
                case HelpCriteria.ConnectionProfile: helpLink += "&page=smarternotify/connectionprofile"; break;
                case HelpCriteria.ConnectionProfileConflict: helpLink += "&page=smarternotify/connectionprofileconflict"; break;
                case HelpCriteria.ImportConflict: helpLink += "&page=smarternotify/importconflict"; break;
                case HelpCriteria.Import: helpLink += "&page=smarternotify/import"; break;
                case HelpCriteria.Export: helpLink += "&page=smarternotify/export"; break;
                case HelpCriteria.MainInterface: helpLink += "&page=smarternotify/maininterface"; break;
                case HelpCriteria.Settings: helpLink += "&page=smarternotify/settings"; break;
                case HelpCriteria.GettingStarted: helpLink += "&page=gettingstarted"; break;
                case HelpCriteria.WelcomeWindow: helpLink += "&page=smarternotify/welcomewindow"; break;
                default: helpLink += "&page=smarternotify/home"; break;
            }

            System.Diagnostics.Process.Start(helpLink);
        }
        public static List<string> GetFormattedEnumNames(Type enumType)
        {
            List<string> retVal = new List<string>();
            List<string> enumNames = new List<string>(Enum.GetNames(enumType));
            if (enumNames != null)
            {
                foreach (string name in enumNames)
                {
                    string nameToAdd = string.Empty;
                    for (int i = 0; i < name.Length; i++)
                    {
                        nameToAdd += name[i];
                        if (name.Length > (i + 1) && char.IsUpper(name[i + 1]))
                            nameToAdd += " ";
                    }
                    retVal.Add(nameToAdd);
                }
            }
            return retVal;
        }
    }
}