﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;
using SmarterNotify.Data;
using SmarterNotify.DataHandlers;
using SmarterNotify.HelperClasses;

namespace SmarterNotify.Threads
{
    public class UpdateThread
    {
        private static UpdateArguments ThreadArguments = new UpdateArguments();
        private static InputArguments ThreadInput = new InputArguments();
        private static bool asyncThreadPerforming = false;
        private static string ThreadLocker = "{C3D9BA86-888A-4d6c-871A-873E9D90D64D}";
        private static string AsyncThreadLocker = "{2A19F7CE-AC75-4f69-9930-B36C5FA40039}";
        public static string NewRemindersLocker = "{357D3A5B-9567-4dbf-9A74-4209C19D0537}";
        public const string INTERNALREMINDERSKEY = "{E06183F2-F7A9-4c5b-860D-2073A5C78EF5}";

        public static void StartUpdateThread()
        {
            Thread thdUpdater = new Thread(new ThreadStart(UpdateThreadMethod));
            thdUpdater.IsBackground = true;
            thdUpdater.Name = "STNotificationToolUpdateThread";
            thdUpdater.Start();
        }
        public static void StartAsyncUpdateThread()
        {
            Thread updateThread = new Thread(new ThreadStart(PerformAsyncUpdate));
            updateThread.IsBackground = true;
            updateThread.Name = "STNotificationToolUpdateAsyncThread";
            updateThread.Start();
        }
        public static void UpdateThreadMethod()
        {
            while (true)
            {
                try
                {
                    UpdateMethod();
                    Thread.Sleep(10000);
                }
                catch (Exception ex)
                {
                    LogHandler.WriteLog(LogHandler.LogType.Error, "Error in update thread", ex.ToString());
                }
            }
        }
        public static void PerformAsyncUpdate()
        {
            AsyncThreadPerforming = true;
            UpdateMethod();
            AsyncThreadPerforming = false;
        }

        public static void UpdateMethod()
        {
            // ping all the associated connection profiles
            foreach (ConnectionProfile profile in ConnectionProfileHandler.GetAllProfiles())
            {
                if (profile.ConnectionStatus == ConnectionProfile.ConnectionStatusType.Started && !profile.IsServiceAvailable() && !profile.IsPendingConnection)
                {
                    profile.IsPendingConnection = true;
                    profile.PauseConnections();
                }
            }

            List<Reminder> allReminders = ReminderHandler.GetAllProfileReminders(ShowSnoozedItems);
            List<Reminder> intReminders = InternalReminderHandler.GetAllInternalReminders(ShowSnoozedItems);
            List<Reminder> plgReminders = InternalReminderHandler.GetAllPluginReminders(ShowSnoozedItems);

            switch (CurrentProfileName)
            {
                case TreeViewHelper.TreeAllNotifications:
                    ActiveReminders = allReminders;
                    ActiveReminders.AddRange(intReminders);
                    break;
                case TreeViewHelper.TreeSmarterNotify:
                    ActiveReminders = intReminders;
                    break;
                case TreeViewHelper.TreeSmarterMail:
                    ActiveReminders = ReminderHandler.GetAllProfileReminders(ShowSnoozedItems, ConnectionProfile.ConnectionProfileType.SmarterMail, allReminders);
                    break;
                case TreeViewHelper.TreeSmarterStats:
                    ActiveReminders = ReminderHandler.GetAllProfileReminders(ShowSnoozedItems, ConnectionProfile.ConnectionProfileType.SmarterStats, allReminders);
                    break;
                case TreeViewHelper.TreeSmarterTrack:
                    ActiveReminders = ReminderHandler.GetAllProfileReminders(ShowSnoozedItems, ConnectionProfile.ConnectionProfileType.SmarterTrack, allReminders);
                    break;
                case TreeViewHelper.TreeExternalProvider:
                    ActiveReminders = ReminderHandler.GetAllProfileReminders(ShowSnoozedItems, ConnectionProfile.ConnectionProfileType.ExternalProvider, allReminders);
                    break;
                case TreeViewHelper.TreePluginProvider:
                    ActiveReminders = plgReminders;
                    break;
                default:
                    ConnectionProfile profile = ConnectionProfileHandler.GetProfile(CurrentProfileName);
                    if (profile != null && profile.ConnectionType == ConnectionProfile.ConnectionProfileType.PluginProvider)
                        ActiveReminders = ReminderHandler.GetAllProfileReminders(ShowSnoozedItems, ConnectionProfile.ConnectionProfileType.PluginProvider, allReminders);
                    else if (profile != null)
                        ActiveReminders = ReminderHandler.GetAllProfileReminders(ShowSnoozedItems, profile, allReminders);
                    break;
            }

            if (allReminders.Count > 0)
            {
                LogHandler.WriteLog(LogHandler.LogType.Notifications, "Reminders detected: " + allReminders.Count.ToString());
            }

            Dictionary<string, int> profileCounts = new Dictionary<string, int>();
            // save some time on # of web service calls - calculate their total counts here
            foreach (Reminder reminder in allReminders)
            {
                ConnectionProfile profile = ConnectionProfileHandler.GetProfile(reminder.ProfileGUID);
                if (profile != null)
                {
                    int numReminders = 0;
                    profileCounts.TryGetValue(profile.ProfileGUID, out numReminders);
                    profileCounts[profile.ProfileGUID] = numReminders + 1;
                }
            }
            // calculate total counts of the internal reminders
            foreach (Reminder reminder in intReminders)
            {
                if (reminder != null)
                {
                    int numReminders = 0;
                    profileCounts.TryGetValue(INTERNALREMINDERSKEY, out numReminders);
                    profileCounts[INTERNALREMINDERSKEY] = numReminders + 1;
                }
            }

            ProfileReminderCounts = profileCounts;
            lock (NewRemindersLocker)
            {
                NewReminders = ViewManager.GetNewNotifications(allReminders, intReminders);
                if (NewReminders.Count > 0)
                {
                    LogHandler.WriteLog(LogHandler.LogType.Notifications, "New Alerts Detected in BG thread", "New alerts have been added to the list to be seen by the main form");
                }
            }
        }

        ///
        /// PROPERTIES (STATIC ACCESSORS)
        /// 

        public static bool AsyncThreadPerforming
        {
            get
            {
                lock (AsyncThreadLocker)
                {
                    return asyncThreadPerforming;
                }
            }
            set
            {
                lock (AsyncThreadLocker)
                {
                    asyncThreadPerforming = value;
                }
            }
        }

        // NumNewAlerts
        public static List<Reminder> NewReminders
        {
            get
            {
                lock (ThreadLocker)
                {
                    return ThreadArguments.NewReminders;
                }
            }
            set
            {
                lock (ThreadLocker)
                {
                    ThreadArguments.NewReminders = value;
                }
            }
        }

        // ActiveReminders
        public static List<Reminder> ActiveReminders
        {
            get
            {
                lock (ThreadLocker)
                {
                    return ThreadArguments.ActiveReminders;
                }
            }
            set
            {
                lock (ThreadLocker)
                {
                    ThreadArguments.ActiveReminders = value;
                }
            }
        }

        // ProfileReminderCounts
        public static Dictionary<string, int> ProfileReminderCounts
        {
            get
            {
                lock (ThreadLocker)
                {
                    return ThreadArguments.ProfileReminderCounts;
                }
            }
            set
            {
                lock (ThreadLocker)
                {
                    ThreadArguments.ProfileReminderCounts = value;
                }
            }
        }

        // CurrentProfileName
        public static string CurrentProfileName
        {
            get
            {
                lock (ThreadLocker)
                {
                    return ThreadInput.CurrentProfileName;
                }
            }
            set
            {
                lock (ThreadLocker)
                {
                    ThreadInput.CurrentProfileName = value;
                }
            }
        }

        // ShowSnoozedItems
        public static bool ShowSnoozedItems
        {
            get
            {
                lock (ThreadLocker)
                {
                    return ThreadInput.ShowSnoozedItems;
                }
            }
            set
            {
                lock (ThreadLocker)
                {
                    ThreadInput.ShowSnoozedItems = value;
                }
            }
        }

        public class UpdateArguments
        {
            public List<Reminder> NewReminders { get; set; }
            public List<Reminder> ActiveReminders { get; set; }
            public Dictionary<string, int> ProfileReminderCounts { get; set; }

            public UpdateArguments()
            {
                NewReminders = new List<Reminder>();
                ActiveReminders = new List<Reminder>();
                ProfileReminderCounts = new Dictionary<string, int>();
            }
        }
        public class InputArguments
        {
            public string CurrentProfileName { get; set; }
            public bool ShowSnoozedItems { get; set; }

            public InputArguments()
            {
                CurrentProfileName = "tvAllNotifications";
                ShowSnoozedItems = false;
            }
        }
    }
}