﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Speech.Synthesis;
using SmarterNotify.Data.Voice;
using SmarterNotify.DataHandlers;

namespace SmarterNotify.HelperClasses
{
    public class VoiceThread
    {
        public static Queue<VoiceQueueItem> VoiceQueue = new Queue<VoiceQueueItem>();
        public static object VoiceLocker = new object();
        private static bool IsQueueProcessing = false;

        public static void AddToQueue(string text)
        {
            lock (VoiceLocker)
            {
                VoiceQueue.Enqueue(new VoiceQueueItem(text, 1000, 0));
                if (!IsQueueProcessing)
                {
                    StartVoiceQueue();
                    IsQueueProcessing = true;
                }
            }
        }
        public static void AddToQueue(string text, int postDelay, int preDelay)
        {
            lock (VoiceLocker)
            {
                VoiceQueue.Enqueue(new VoiceQueueItem(text, postDelay, preDelay));
                if (!IsQueueProcessing)
                {
                    StartVoiceQueue();
                    IsQueueProcessing = true;
                }
            }
        }
        public static void ClearQueue()
        {
            lock (VoiceLocker)
            {
                VoiceQueue.Clear();
            }
        }
        public static bool IsQueueProcessing_Safe()
        {
            lock (VoiceLocker)
            {
                return IsQueueProcessing;
            }
        }
        private static void StartVoiceQueue()
        {
            Thread thd = new Thread(new ThreadStart(CycleVoiceQueue));
            thd.IsBackground = true;
            thd.Name = "VoiceQueueCycleThroughAndPlay";
            thd.Start();
        }
        private static void CycleVoiceQueue()
        {
            while (VoiceQueue.Count > 0)
            {
                string message = string.Empty;
                try
                {
                    int postDelay = 0;
                    int preDelay = 0;
                    VoiceQueueItem queueItm = new VoiceQueueItem();
                    lock (VoiceLocker)
                    {
                        queueItm = VoiceQueue.Dequeue();
                        message = queueItm.Message;
                        postDelay = queueItm.PostDelay;
                        preDelay = queueItm.PreDelay;
                    }

                    if (preDelay > 0)
                        Thread.Sleep(preDelay);
                    
                    using (var synth = new SpeechSynthesizer())
                    {
                        synth.Speak(VoiceDictionaryHandler.PhonicConvert(message));
                    }
                    
                    if (postDelay > 0)
                        Thread.Sleep(postDelay);
                }
                catch (Exception ex)
                {
                    string logMsg = "Error occurred in the voice queue while reading \"" + message + "\" into a voice message";
                    TrackingHelper.TrackError(TrackingHelper.RecordType.Both, "Error Occurred", logMsg, logMsg, ex);
                }
            }
            lock (VoiceLocker)
            {
                IsQueueProcessing = false;
            }
        }
    }
}