﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using SmarterNotify.Data;
using SmarterNotify.HelperClasses;

namespace SmarterNotify
{
    public partial class frmImportConflict : Form
    {
        public ConnectionProfile ImportingProfile = new ConnectionProfile();
        public ConnectionProfile CurrentProfile = new ConnectionProfile();
        public bool ActionComitted = false;

        public enum ActionType
        {
            Null,
            Skip,
            Add,
            Replace
        }

        public delegate void OnActionDel(ConnectionProfile importingProfile, ActionType type);
        public event OnActionDel OnAction;

        public delegate void OnApplyAllDel(ActionType applyAllType);
        public event OnApplyAllDel OnApplyAll;

        public frmImportConflict()
        {
            InitializeComponent();
        }
        private void frmImportWarning_Load(object sender, EventArgs e)
        {
            // setup the importing profile info
            if (ImportingProfile != null)
            {
                lblImpAuthPassword.Text = ImportingProfile.AuthPassword;
                lblImpAuthUsername.Text = ImportingProfile.AuthUsername;
                lblImpConnectAppStart.Text = ImportingProfile.StartAppConnected == true ? "Yes" : "No";
                lblImpConnectionURL.Text = ImportingProfile.ConnectionURL;
                lblImpDataUsername.Text = ImportingProfile.DataUsername;
                lblImpProfileName.Text = ImportingProfile.ProfileName;
                lblImpProvider.Text = ImportingProfile.ConnectionType.ToString();
            }

            // setup the current profile info
            if (CurrentProfile != null)
            {
                lblCurAuthPassword.Text = CurrentProfile.AuthPassword;
                lblCurAuthUsername.Text = CurrentProfile.AuthUsername;
                lblCurConnectAppStart.Text = CurrentProfile.StartAppConnected == true ? "Yes" : "No";
                lblCurConnectionURL.Text = CurrentProfile.ConnectionURL;
                lblCurDataUsername.Text = CurrentProfile.DataUsername;
                lblCurProfileName.Text = CurrentProfile.ProfileName;
                lblCurProvider.Text = CurrentProfile.ConnectionType.ToString();
            }
        }
        private void btnSkip_Click(object sender, EventArgs e)
        {
            ActionType type = ActionType.Skip;
            OnAction(ImportingProfile, type);
            CloseForm(type);
        }
        private void btnIgnore_Click(object sender, EventArgs e)
        {
            ActionType type = ActionType.Add;
            OnAction(ImportingProfile, type);
            CloseForm(type);
        }
        private void btnReplace_Click(object sender, EventArgs e)
        {
            ActionType type = ActionType.Replace;
            OnAction(ImportingProfile, type);
            CloseForm(type);
        }
        private void CloseForm(ActionType type)
        {
            if (chkApplyToAll.Checked)
                OnApplyAll(type);
            ActionComitted = true;
            this.Close();
        }
        private void frmImportConflict_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (e.CloseReason == CloseReason.UserClosing && !ActionComitted)
            {
                // if user tries to just close the window... perform ignore action
                OnAction(ImportingProfile, ActionType.Add);
            }
        }
        private void btnHelp_Click(object sender, EventArgs e)
        {
            ViewManager.OpenHelp(ViewManager.HelpCriteria.ImportConflict);
        }
        private void btnHelp_HelpRequested(object sender, HelpEventArgs hlpevent)
        {
            ViewManager.OpenHelp(ViewManager.HelpCriteria.ImportConflict);
        }
    }
}