﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using SmarterNotify.DataHandlers;
using SmarterNotify.Data;
using SmarterNotify.HelperClasses;
using System.Linq;

namespace SmarterNotify
{
    public partial class frmImportExport : Form
    {
        public enum DataInteractionType
        {
            Import,
            Export
        }
        public DataInteractionType DataType = DataInteractionType.Import;
        public List<ConnectionProfile> AvailableProfiles = new List<ConnectionProfile>();
        public frmImportConflict.ActionType ApplyToAllType = frmImportConflict.ActionType.Null;

        public delegate void UpdateParentList();
        public event UpdateParentList OnUpdate;

        public frmImportExport()
        {
            InitializeComponent();
        }
        private void frmImportExport_Load(object sender, EventArgs e)
        {
            if (DataType == DataInteractionType.Export)
            {
                btnData.Text = "Export";
                this.Text = "Export Connection Profiles";
                SetGVDataSource(ConnectionProfileHandler.GetAllProfiles());
            }
            else
            {
                btnData.Text = "Import";
                this.Text = "Import Connection Profiles";
            }
        }
        private void btnBrowse_Click(object sender, EventArgs e)
        {
            DialogResult result = DialogResult.Cancel;
            if (DataType == DataInteractionType.Import)
            {
                string browsedText = txtBrowse.Text;
                result = opnFile.ShowDialog();
                if (result == DialogResult.OK || result == DialogResult.Yes)
                {
                    browsedText = opnFile.FileName;
                    // fill the grid with the selected files content
                    SetGVDataSource(ConnectionProfileHandler.GetProfilesFromFile(browsedText));
                    txtBrowse.Text = browsedText;
                }
            }
            else
            {
                result = opnDirectory.ShowDialog();
                if (result == DialogResult.OK || result == DialogResult.Yes)
                    txtBrowse.Text = opnDirectory.SelectedPath;
            }
        }
        private void btnData_Click(object sender, EventArgs e)
        {
            try
            {
                string fileBrowsed = txtBrowse.Text;
                if (fileBrowsed.Trim().Length == 0)
                {
                    if (DataType == DataInteractionType.Import)
                    {
                        MessageBox.Show("You must select a file to import",
                            "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                    else if (DataType == DataInteractionType.Export)
                    {
                        MessageBox.Show("You must select a destination for the selected exported data",
                            "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                    return;
                }

                if (DataType == DataInteractionType.Import)
                {
                    foreach (ConnectionProfile profile in SelectedProfiles)
                    {
                        // check to make sure this profile's GUID doesn't already exist in another profile
                        ConnectionProfile currentProfile = ConnectionProfileHandler.GetProfile(profile.ProfileGUID);
                        if (currentProfile != null)
                        {
                            if (ApplyToAllType == frmImportConflict.ActionType.Null)
                            {
                                // profile exists... show conflict window
                                frmImportConflict conflict = new frmImportConflict();
                                conflict.TopMost = false;
                                conflict.ImportingProfile = profile;
                                conflict.CurrentProfile = currentProfile;
                                conflict.OnAction += new frmImportConflict.OnActionDel(conflict_OnAction);
                                conflict.OnApplyAll += new frmImportConflict.OnApplyAllDel(conflict_OnApplyAll);
                                conflict.ShowDialog();
                            }
                            else
                                conflict_OnAction(profile, ApplyToAllType);
                        }
                        else
                        {
                            // create new profile
                            ConnectionProfileHandler.SaveProfile(profile, false);
                        }
                    }

                    ConnectionProfileHandler.WriteProfilesToFile();
                }
                else
                {
                    // create exported file with timestamp
                    if (!Directory.Exists(fileBrowsed))
                        Directory.CreateDirectory(fileBrowsed);
                    if (!fileBrowsed.EndsWith("\\"))
                        fileBrowsed += "\\";

                    string dtUniquifier = DateTime.Now.ToShortDateString().Replace("/", "-") + "-" + DateTime.Now.Hour + DateTime.Now.Minute + DateTime.Now.Second;
                    string fileName = fileBrowsed + "SmarterNotify Connections " + dtUniquifier + ".xml";

                    if (File.Exists(fileName))
                    {
                        int numCopy = 2;
                        while (File.Exists(fileName))
                        {
                            fileName = fileBrowsed + "SmarterNotify Connections " + dtUniquifier + " (" + numCopy + ").xml";
                            numCopy++;
                        }
                    }
                    ConnectionProfileHandler.WriteProfilesToFile(fileName, SelectedProfiles);
                }
            }
            catch (Exception ex) 
            {
                string type = DataType == DataInteractionType.Import ? "importing" : "exporting";

                string logMsg = "Error occurred while " + type + "the current connection profiles";
                TrackingHelper.TrackError(TrackingHelper.RecordType.Both, "Error Occurred", logMsg, logMsg, ex);
            }
            OnUpdate();
            this.Close();
        }
        private void btnCancel_Click(object sender, EventArgs e)
        {
            this.Close();
        }
        private void chkSelectAll_CheckedChanged(object sender, EventArgs e)
        {
            bool isSelectAll = chkSelectAll.Checked;
            foreach (DataGridViewRow row in gvAllProfiles.Rows)
                row.Selected = isSelectAll;
            //else 
            if (!isSelectAll && gvAllProfiles.SelectedRows.Count == 0 && gvAllProfiles.Rows.Count > 0)
                SelectCurrentCellRow();
        }
        private void gvAllProfiles_MouseClick(object sender, MouseEventArgs e)
        {
            if (gvAllProfiles.SelectedRows.Count == gvAllProfiles.Rows.Count)
            {
                if (chkSelectAll.Enabled)
                    chkSelectAll.Checked = true;
            }
            else
            {
                List<int> rowIndices = new List<int>();
                foreach (DataGridViewRow row in gvAllProfiles.SelectedRows)
                    rowIndices.Add(row.Index);
                if (chkSelectAll.Enabled)
                    chkSelectAll.Checked = false;
                foreach (int index in rowIndices)
                    gvAllProfiles.Rows[index].Selected = true;
                if (gvAllProfiles.SelectedRows.Count == 0 && gvAllProfiles.Rows.Count > 0)
                    SelectCurrentCellRow();
            }
        }
        private void btnHelp_Click(object sender, EventArgs e)
        {
            switch (DataType)
            {
                case DataInteractionType.Import: ViewManager.OpenHelp(ViewManager.HelpCriteria.Import); break;
                case DataInteractionType.Export: ViewManager.OpenHelp(ViewManager.HelpCriteria.Export); break;
                default: ViewManager.OpenHelp(ViewManager.HelpCriteria.HomePage); break;
            }
        }
        private void frmImportExport_HelpRequested(object sender, HelpEventArgs hlpevent)
        {
            switch (DataType)
            {
                case DataInteractionType.Import: ViewManager.OpenHelp(ViewManager.HelpCriteria.Import); break;
                case DataInteractionType.Export: ViewManager.OpenHelp(ViewManager.HelpCriteria.Export); break;
                default: ViewManager.OpenHelp(ViewManager.HelpCriteria.HomePage); break;
            }
        }

        #region Helper Methods
        public List<ConnectionProfile> SelectedProfiles
        {
            get
            {
                try
                {
                    List<ConnectionProfile> selectedProfiles = new List<ConnectionProfile>();
                    foreach (ConnectionProfile profile in AvailableProfiles)
                    {
                        foreach (DataGridViewRow row in gvAllProfiles.SelectedRows)
                        {
                            object profileGUID = row.Cells["colProfileGUID"].Value;
                            if (profileGUID != null && profileGUID.ToString() == profile.ProfileGUID)
                            {
                                selectedProfiles.Add(profile);
                                break;
                            }
                        }
                    }
                    return selectedProfiles;
                }
                catch
                {
                    return new List<ConnectionProfile>();
                }
            }
        }
        public void SetGVDataSource(List<ConnectionProfile> profiles)
        {
            List<ConnectionProfile> availableProfiles = new List<ConnectionProfile>();
            foreach (ConnectionProfile profile in profiles)
                if (profile.ConnectionType != ConnectionProfile.ConnectionProfileType.PluginProvider)
                    availableProfiles.Add(profile);

            AvailableProfiles = availableProfiles;
            gvAllProfiles.DataSource = availableProfiles;
            if (gvAllProfiles.Rows.Count == 1)
                chkSelectAll.Enabled = false;
            else
            {
                chkSelectAll.Enabled = true;
                chkSelectAll_CheckedChanged(null, null);
            }
        }
        public void conflict_OnAction(ConnectionProfile importingProfile, frmImportConflict.ActionType type)
        {
            frmImportConflict.ActionType usedType = type;
            if (ApplyToAllType != frmImportConflict.ActionType.Null)
                usedType = ApplyToAllType;

            switch (usedType)
            {
                case frmImportConflict.ActionType.Add:
                    ConnectionProfile newProfile = new ConnectionProfile();
                    newProfile.AuthPassword = importingProfile.AuthPassword;
                    newProfile.AuthUsername = importingProfile.AuthUsername;
                    newProfile.ConnectionType = importingProfile.ConnectionType;
                    newProfile.ConnectionURL = importingProfile.ConnectionURL;
                    newProfile.DataUsername = importingProfile.DataUsername;
                    newProfile.ProfileName = importingProfile.ProfileName;
                    newProfile.StartAppConnected = importingProfile.StartAppConnected;
                    ConnectionProfileHandler.SaveProfile(newProfile, false);
                    break;
                case frmImportConflict.ActionType.Replace:
                    ConnectionProfileHandler.SaveProfile(importingProfile, false);
                    break;
                case frmImportConflict.ActionType.Skip: 
                    break;
            }
        }
        public void conflict_OnApplyAll(frmImportConflict.ActionType type)
        {
            ApplyToAllType = type;
        }
        private void SelectCurrentCellRow()
        {
            int selectableIndex = gvAllProfiles.CurrentCell.RowIndex;
            if (gvAllProfiles.Rows.Count > selectableIndex)
                gvAllProfiles.Rows[selectableIndex].Selected = true;
        }
        #endregion
    }
}