﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using SmarterNotify.DataHandlers;
using SmarterNotify.Data;
using SmarterNotify.HelperClasses;
using System.IO;
using System.Threading;
using System.Reflection;
using SmarterNotify.Connector;
using System.Speech;
using System.Speech.Synthesis;
using System.Globalization;
using SmarterNotify.Threads;
using System.Diagnostics;

namespace SmarterNotify
{
    public partial class frmMain : Form
    {
        public frmMain()
        {
            InitializeComponent();
            LoadConfigFiles();
        }
        public enum TaskbarIconTypes
        {
            Normal,
            Alert
        }
        public enum BalloonTypes
        {
            Minimize,
            Notify
        }
        public List<int> CurrentReminderIndices = new List<int>();
        public TaskbarIconTypes NextTaskbarIcon = TaskbarIconTypes.Normal;
        public bool MultipleTVSelected = false;
        public string RemindersKey = "";
        public BalloonTypes BalloonTypeAction = BalloonTypes.Notify;
        public bool HasFormHidden = false;

        private void frmMain_Load(object sender, EventArgs e)
        {
            gvNotifications.AutoGenerateColumns = false;
            UpdateThread.CurrentProfileName = TreeViewHelper.TreeAllNotifications;
            UpdateThread.UpdateMethod();
            ShowDefaultSetup();
            ViewManager.UpdateLastCheckDate();
            UpdateThread.StartUpdateThread();
            ShowNotificationContent();
        }
        private void frmMain_Shown(object sender, EventArgs e)
        {
            // see if the welcome window should show
            if (SettingsHandler.GlobalSettings.IsFirstLoad)
            {
                frmFirstLoad welcomeForm = new frmFirstLoad();
                welcomeForm.Size = new Size(469, 238);
                welcomeForm.TopMost = false;
                welcomeForm.ShowDialog();
            }
            if (WindowStateManager.CurrentState == WindowStateManager.StateTypes.Normal)
            {
                this.gvNotifications.Focus();
                StackTrace st = new StackTrace(true);
                LogHandler.WriteLog(LogHandler.LogType.Error, "Window focusing " + st.ToString());
            }
        }
        private void btnDismissAll_Click(object sender, EventArgs e)
        {
            DismissAllReminders(UpdateThread.CurrentProfileName);
            UpdateWorkArea(true);
        }
        private void mnuDismissAll_Click(object sender, EventArgs e)
        {
            DismissAllReminders(UpdateThread.CurrentProfileName);
            UpdateWorkArea(true);
        }
        private void itmDismissAll_Click(object sender, EventArgs e)
        {
            DismissAllReminders(UpdateThread.CurrentProfileName);
            UpdateWorkArea(true);
        }
        private void btnDismiss_Click(object sender, EventArgs e)
        {
            DismissSelectedReminders();
            UpdateWorkArea(true);
        }
        private void mnuDismissSelected_Click(object sender, EventArgs e)
        {
            DismissSelectedReminders();
            UpdateWorkArea(true);
        }
        private void mnuNewConnectionDD_Click(object sender, EventArgs e)
        {
            ShowNewProfileWindow();
            UpdateWorkArea(true);
        }
        private void mnuNewConnection_Click(object sender, EventArgs e)
        {
            ShowNewProfileWindow();
            UpdateWorkArea(true);
        }
        private void mnuEditConnection_Click(object sender, EventArgs e)
        {
            ShowEditProfileWindow(UpdateThread.CurrentProfileName);
            UpdateWorkArea(true);
        }
        private void mnuEditConnectionDD_Click(object sender, EventArgs e)
        {
            ShowEditProfileWindow(UpdateThread.CurrentProfileName);
            UpdateWorkArea(true);
        }
        private void mnuDeleteConnection_Click(object sender, EventArgs e)
        {
            DeleteProfile(UpdateThread.CurrentProfileName);
        }
        private void mnuDeleteConnectionDD_Click(object sender, EventArgs e)
        {
            DeleteProfile(UpdateThread.CurrentProfileName);
        }
        private void mnuRefresh_Click(object sender, EventArgs e)
        {
            GenerateConnectionsArea();
        }
        private void mnuRefreshDD_Click(object sender, EventArgs e)
        {
            GenerateConnectionsArea();
        }

        #region Snooze Functionality
        private void btnSnooze_Click(object sender, EventArgs e)
        {
            SnoozeSelectedReminders();
            UpdateWorkArea(true);
        }
        private void mnuSnooze15_Click(object sender, EventArgs e)
        {
            SnoozeSelectedReminders(900);
            UpdateWorkArea(true);
        }
        private void itmSnooze15_Click(object sender, EventArgs e)
        {
            SnoozeAllReminders(900);
            UpdateWorkArea(true);
        }
        private void mnuSnooze30_Click(object sender, EventArgs e)
        {
            SnoozeSelectedReminders(1800);
            UpdateWorkArea(true);
        }
        private void itmSnooze30_Click(object sender, EventArgs e)
        {
            SnoozeAllReminders(1800);
            UpdateWorkArea(true);
        }
        private void mnuSnooze1Hour_Click(object sender, EventArgs e)
        {
            SnoozeSelectedReminders(3600);
            UpdateWorkArea(true);
        }
        private void itmSnooze1Hour_Click(object sender, EventArgs e)
        {
            SnoozeAllReminders(3600);
            UpdateWorkArea(true);
        }
        private void mnuSnooze2Hours_Click(object sender, EventArgs e)
        {
            SnoozeSelectedReminders(7200);
            UpdateWorkArea(true);
        }
        private void itmSnooze2Hours_Click(object sender, EventArgs e)
        {
            SnoozeAllReminders(7200);
            UpdateWorkArea(true);
        }
        private void mnuSnooze3Hours_Click(object sender, EventArgs e)
        {
            SnoozeSelectedReminders(10800);
            UpdateWorkArea(true);
        }
        private void itmSnooze3Hours_Click(object sender, EventArgs e)
        {
            SnoozeAllReminders(10800);
            UpdateWorkArea(true);
        }
        private void mnuSnooze4Hours_Click(object sender, EventArgs e)
        {
            SnoozeSelectedReminders(14400);
            UpdateWorkArea(true);
        }
        private void itmSnooze4Hours_Click(object sender, EventArgs e)
        {
            SnoozeAllReminders(14400);
            UpdateWorkArea(true);
        }
        private void mnuSnooze5Hours_Click(object sender, EventArgs e)
        {
            SnoozeSelectedReminders(18000);
            UpdateWorkArea(true);
        }
        private void itmSnooze5Hours_Click(object sender, EventArgs e)
        {
            SnoozeAllReminders(18000);
            UpdateWorkArea(true);
        }
        private void mnuSnooze6Hours_Click(object sender, EventArgs e)
        {
            SnoozeSelectedReminders(21600);
            UpdateWorkArea(true);
        }
        private void itmSnooze6Hours_Click(object sender, EventArgs e)
        {
            SnoozeAllReminders(21600);
            UpdateWorkArea(true);
        }
        private void mnuSnooze12Hours_Click(object sender, EventArgs e)
        {
            SnoozeSelectedReminders(43200);
            UpdateWorkArea(true);
        }
        private void itmSnooze12Hours_Click(object sender, EventArgs e)
        {
            SnoozeAllReminders(43200);
            UpdateWorkArea(true);
        }
        private void mnuSnooze1Day_Click(object sender, EventArgs e)
        {
            SnoozeSelectedReminders(86400);
            UpdateWorkArea(true);
        }
        private void itmSnooze1Day_Click(object sender, EventArgs e)
        {
            SnoozeAllReminders(86400);
            UpdateWorkArea(true);
        }
        private void mnuSnooze2Days_Click(object sender, EventArgs e)
        {
            SnoozeSelectedReminders(172800);
            UpdateWorkArea(true);
        }
        private void itmSnooze2Days_Click(object sender, EventArgs e)
        {
            SnoozeAllReminders(172800);
            UpdateWorkArea(true);
        }
        private void mnuSnooze3Days_Click(object sender, EventArgs e)
        {
            SnoozeSelectedReminders(259200);
            UpdateWorkArea(true);
        }
        private void itmSnooze3Days_Click(object sender, EventArgs e)
        {
            SnoozeAllReminders(259200);
            UpdateWorkArea(true);
        }
        #endregion

        private void mnuConnectSelected_Click(object sender, EventArgs e)
        {
            ToggleConnection(true);
            UpdateWorkArea(true);
        }
        private void mnuConnect_Click(object sender, EventArgs e)
        {
            ToggleConnection(true);
            UpdateWorkArea(true);
        }
        private void mnuConnectDD_Click(object sender, EventArgs e)
        {
            ToggleConnection(true);
            UpdateWorkArea(true);
        }
        private void mnuDisconnectSelected_Click(object sender, EventArgs e)
        {
            ToggleConnection(false);
            UpdateWorkArea(true);
        }
        private void mnuDisconnect_Click(object sender, EventArgs e)
        {
            ToggleConnection(false);
            UpdateWorkArea(true);
        }
        private void mnuDisconnectDD_Click(object sender, EventArgs e)
        {
            ToggleConnection(false);
            UpdateWorkArea(true);
        }
        private void itmPlaySounds_Click(object sender, EventArgs e)
        {
            SettingsHandler.GlobalSettings.NotifyPlaySound = itmPlaySounds.Checked;
            SettingsHandler.SaveSettings();
        }
        private void mnuImportProfiles_Click(object sender, EventArgs e)
        {
            frmImportExport dataForm = new frmImportExport();
            dataForm.DataType = frmImportExport.DataInteractionType.Import;
            dataForm.OnUpdate += new frmImportExport.UpdateParentList(GenerateConnectionsArea);
            dataForm.TopMost = false;
            dataForm.ShowDialog();
        }
        private void mnuExportProfiles_Click(object sender, EventArgs e)
        {
            frmImportExport dataForm = new frmImportExport();
            dataForm.DataType = frmImportExport.DataInteractionType.Export;
            dataForm.OnUpdate += new frmImportExport.UpdateParentList(GenerateConnectionsArea);
            dataForm.TopMost = false;
            dataForm.ShowDialog();
        }

        #region Helper Methods
        private void UpdateWorkArea(bool updateThread)
        {
            if (updateThread)
            {
                if (!UpdateThread.AsyncThreadPerforming)
                    UpdateThread.StartAsyncUpdateThread();
                else
                    MultipleTVSelected = true;

                tmrAsyncRequestChecker.Enabled = true;
            }
            else
                UpdateWorkAreaSub();
        }
        private void UpdateWorkAreaSub()
        {
            gvNotifications.Columns["colAllSnoozedUntil"].Visible = UpdateThread.ShowSnoozedItems;
            lock (RemindersKey)
            {
                List<Reminder> reminders = UpdateThread.ActiveReminders;
                reminders.Sort(Reminder.TimeOccurredComparisonDESC);
                gvNotifications.DataSource = reminders;
                // remove first index at first try, let the selector handle this
                if (CurrentReminderIndices.Count > 0)
                {
                    foreach (DataGridViewRow row in gvNotifications.Rows)
                        row.Selected = false;
                }
            }
            SelectCurrentIndices();
            ShowNotificationContent();
            TreeViewHelper.UpdateTreeNodes(ref tvConnections);
        }
        private void GenerateConnectionsArea()
        {
            itmPlaySounds.Checked = SettingsHandler.GlobalSettings.NotifyPlaySound;
            UpdateThread.UpdateMethod();
            TreeViewHelper.GenerateTreeNodes(ref tvConnections);
            UpdateWorkArea(false);
        }
        private void ShowNewProfileWindow()
        {
            frmConnectionProfile profile = new frmConnectionProfile("");
            profile.OnUpdate += new frmConnectionProfile.UpdateParentList(GenerateConnectionsArea);
            profile.TopMost = false;
            profile.ShowDialog();
        }
        private void ShowEditProfileWindow(string profileID)
        {
            if (!IsReservedName(profileID))
            {
                frmConnectionProfile profile = new frmConnectionProfile(profileID);
                profile.OnUpdate += new frmConnectionProfile.UpdateParentList(GenerateConnectionsArea);
                profile.TopMost = false;
                profile.ShowDialog();
            }
        }
        private void DeleteProfile(string profileID)
        {
            if (!IsReservedName(profileID))
            {
                ConnectionProfileHandler.DeleteProfile(profileID);
                UpdateThread.CurrentProfileName = TreeViewHelper.TreeAllNotifications;
                GenerateConnectionsArea();
            }

            ClearSelectedRowIndices();
        }
        private void DismissAllReminders(string profileID)
        {
            ConnectionProfile usedProfile = new ConnectionProfile();
            if (IsInternalProfile(profileID))
                InternalReminderHandler.DismissAllInternalReminders(true);
            else
            {
                if (!IsReservedName(profileID))
                {
                    usedProfile = ConnectionProfileHandler.GetProfile(profileID);
                    if (usedProfile.ConnectionType != ConnectionProfile.ConnectionProfileType.PluginProvider &&
                        usedProfile.ConnectionStatus != ConnectionProfile.ConnectionStatusType.Started)
                    {
                        ShowConnectedMB();
                        return;
                    }
                    usedProfile.DismissAllReminders();
                }
                else
                {
                    switch (profileID)
                    {
                        case TreeViewHelper.TreeAllNotifications:
                            ReminderHandler.DismissAllProfilesReminders();
                            InternalReminderHandler.DismissAllInternalReminders(true);
                            InternalReminderHandler.DismissAllPluginReminders(true);
                            break;
                        case TreeViewHelper.TreeSmarterMail: ReminderHandler.DismissAllSMProfilesReminders(); break;
                        case TreeViewHelper.TreeSmarterTrack: ReminderHandler.DismissAllSTProfilesReminders(); break;
                        case TreeViewHelper.TreeSmarterStats: ReminderHandler.DismissAllSSProfilesReminders(); break;
                        case TreeViewHelper.TreeExternalProvider: ReminderHandler.DismissAllEPProfilesReminders(); break;
                        case TreeViewHelper.TreePluginProvider: ReminderHandler.DismissAllPlgProfilesReminders(); break;
                    }
                }
            }

            ClearSelectedRowIndices();
            ClearNoteStatus();

            if (SettingsHandler.GlobalSettings.MinimizeOnDismissAll && WindowStateManager.CurrentState != WindowStateManager.StateTypes.Minimized)
                HideForm();
        }
        private void DismissSelectedReminders()
        {
            // <profileGUID, profile>
            Dictionary<string, ConnectionProfile> usedProfiles = new Dictionary<string, ConnectionProfile>();
            bool isAnyInternals = false;
            foreach (Reminder reminder in SelectedReminders)
            {
                if (reminder == null)
                    continue;

                if (reminder.IsInternalOrPlugin())
                {
                    if (!isAnyInternals)
                        isAnyInternals = true;
                    InternalReminderHandler.DismissReminder(reminder.ReminderID, false);
                }
                else
                {
                    ConnectionProfile usedProfile = new ConnectionProfile();
                    if (usedProfiles.TryGetValue(reminder.ProfileGUID, out usedProfile))
                    {
                        if (usedProfile.ConnectionStatus != ConnectionProfile.ConnectionStatusType.Started)
                        {
                            ShowConnectedMB();
                            break;
                        }
                        if (!reminder.IsInternalOrPlugin())
                            usedProfile.DissmissReminder(reminder.ReminderID);
                        else
                            InternalReminderHandler.DismissReminder(reminder.ReminderID, false);
                        continue;
                    }
                    else
                    {
                        usedProfiles[reminder.ProfileGUID] = ConnectionProfileHandler.GetProfile(reminder.ProfileGUID);
                        usedProfiles[reminder.ProfileGUID].DissmissReminder(reminder.ReminderID);
                    }
                }
            }

            if (isAnyInternals)
                InternalReminderHandler.WriteRemindersToFile();

            ClearSelectedRowIndices();
        }
        private void SnoozeAllReminders(int snoozedSeconds)
        {
            SnoozeMethod(snoozedSeconds, ReminderHandler.GetAllProfileReminders(false));
        }
        private void SnoozeSelectedReminders()
        {
            SnoozeSelectedReminders(SelectedSnoozeSeconds);
        }
        private void SnoozeSelectedReminders(int snoozedSeconds)
        {
            SnoozeMethod(snoozedSeconds, SelectedReminders);
        }
        private void SnoozeMethod(int snoozedSeconds, List<Reminder> reminders)
        {
            Dictionary<string, ConnectionProfile> usedProfiles = new Dictionary<string, ConnectionProfile>();
            bool isAnyInternals = false;
            foreach (Reminder reminder in reminders)
            {
                if (reminder == null || string.IsNullOrEmpty(reminder.ProfileGUID))
                    continue;

                if (!reminder.IsInternalOrPlugin())
                {
                    ConnectionProfile usedProfile = new ConnectionProfile();
                    if (usedProfiles.TryGetValue(reminder.ProfileGUID, out usedProfile))
                    {
                        if (usedProfile.ConnectionStatus != ConnectionProfile.ConnectionStatusType.Started)
                        {
                            ShowConnectedMB();
                            break;
                        }

                        usedProfile.SnoozeReminder(reminder.ReminderID, snoozedSeconds);
                        continue;
                    }
                    else
                    {
                        usedProfiles[reminder.ProfileGUID] = ConnectionProfileHandler.GetProfile(reminder.ProfileGUID);
                        usedProfiles[reminder.ProfileGUID].SnoozeReminder(reminder.ReminderID, snoozedSeconds);
                    }
                }
                else
                {
                    if (!isAnyInternals)
                        isAnyInternals = true;
                    InternalReminderHandler.SnoozeReminder(reminder.ReminderID, snoozedSeconds, false);
                }
            }

            if (isAnyInternals)
                InternalReminderHandler.WriteRemindersToFile();

            ClearSelectedRowIndices();
            ClearNoteStatus();
        }
        public List<Reminder> SelectedReminders
        {
            get
            {
                try
                {
                    List<Reminder> selectedReminderList = new List<Reminder>();
                    foreach (int rowToGetData in CurrentReminderIndices)
                    {
                        if (gvNotifications.Rows.Count > rowToGetData && rowToGetData > -1)
                        {
                            DataGridViewRow row = gvNotifications.Rows[rowToGetData];
                            object reminderID = row.Cells["colReminderID"].Value;
                            object profileID = row.Cells["colProfileGUID"].Value;
                            Reminder reminderToAdd = new Reminder();
                            if (reminderID != null && profileID != null)
                            {
                                string profileIDParsed = profileID.ToString();
                                if (string.IsNullOrEmpty(profileIDParsed))
                                    reminderToAdd = InternalReminderHandler.GetInternalReminder(reminderID.ToString());
                                else
                                {
                                    ConnectionProfile profile = ConnectionProfileHandler.GetProfile(profileIDParsed);
                                    reminderToAdd = profile.GetReminder(reminderID.ToString());
                                }
                            }
                            selectedReminderList.Add(reminderToAdd);
                        }
                    }
                    return selectedReminderList;
                }
                catch
                {
                    return new List<Reminder>();
                }
            }
        }
        public int SelectedSnoozeSeconds
        {
            get
            {
                int retVal = -1;
                try
                {
                    if (cboSnoozeTimeFrame.SelectedItem != null)
                    {
                        switch (cboSnoozeTimeFrame.SelectedItem.ToString().ToUpperInvariant())
                        {
                            case "1 MINUTE": retVal = 60; break;
                            case "15 MINUTES": retVal = 900; break;
                            case "30 MINUTES": retVal = 1800; break;
                            case "1 HOUR": retVal = 3600; break;
                            case "2 HOURS": retVal = 7200; break;
                            case "3 HOURS": retVal = 10800; break;
                            case "4 HOURS": retVal = 14400; break;
                            case "5 HOURS": retVal = 18000; break;
                            case "6 HOURS": retVal = 21600; break;
                            case "12 HOURS": retVal = 43200; break;
                            case "1 DAY": retVal = 86400; break;
                            case "2 DAYS": retVal = 172800; break;
                            case "3 DAYS": retVal = 259200; break;
                            default: retVal = 900; break;
                        }
                    }
                }
                catch { }
                return retVal;
            }
        }
        private void ShowNotificationContent()
        {
            int rowToGetData = -1;
            if (CurrentReminderIndices.Count > 0)
                rowToGetData = CurrentReminderIndices[0];

            bool textSet = false;
            if (gvNotifications.Rows.Count > rowToGetData && rowToGetData > -1)
            {
                DataGridViewRow row = gvNotifications.Rows[rowToGetData];
                object reminderType = row.Cells["colReminderType"].Value;
                object reminderID = row.Cells["colReminderID"].Value;
                object profileID = row.Cells["colProfileGUID"].Value;

                Reminder.AlertType remType = Reminder.AlertType.WebService;
                if (reminderType != null)
                {
                    switch (reminderType.ToString().ToUpperInvariant())
                    {
                        case "INTERNAL": remType = Reminder.AlertType.Internal; break;
                        case "PLUGIN": remType = Reminder.AlertType.Plugin; break;
                    }
                }
                if (remType == Reminder.AlertType.Internal && reminderID != null)
                {
                    try
                    {
                        Reminder reminder = InternalReminderHandler.GetInternalReminder(reminderID.ToString());
                        if (reminder != null)
                        {
                            BrowserHandler.LoadBrowserNotification(ref webNotification, reminder.BodyText);
                            textSet = true;
                        }
                    }
                    catch (Exception ex)
                    {
                        string logMsg = "Error occurred while getting the notification message for internal reminder " + reminderID.ToString() + ")";
                        TrackingHelper.TrackError(TrackingHelper.RecordType.Both, "Error Occurred", logMsg, logMsg, ex);
                    }
                }
                else
                {
                    if (reminderID != null && profileID != null)
                    {
                        try
                        {
                            ConnectionProfile profile = ConnectionProfileHandler.GetProfile(profileID.ToString());
                            if (profile != null)
                            {
                                if (reminderID.ToString().Trim().Length > 0)
                                {
                                    BrowserHandler.LoadBrowserNotification(ref webNotification, profile.GetReminder(reminderID.ToString()).BodyText);
                                    textSet = true;
                                }
                            }
                        }
                        catch (Exception ex)
                        {
                            string logMsg = "Error occurred while getting the notification message for reminder (" + reminderID.ToString() + ")";
                            TrackingHelper.TrackError(TrackingHelper.RecordType.Both, "Error Occurred", logMsg, logMsg, ex);
                        }
                    }
                }
            }
            if (!textSet)
                BrowserHandler.ClearBrowser(ref webNotification);
        }
        private bool IsReservedName(string name)
        {
            if (name == TreeViewHelper.TreeAllNotifications ||
                name == TreeViewHelper.TreeSmarterMail ||
                name == TreeViewHelper.TreeSmarterTrack ||
                name == TreeViewHelper.TreeSmarterStats ||
                name == TreeViewHelper.TreeExternalProvider ||
                name == TreeViewHelper.TreePluginProvider)
                return true;

            return false;
        }
        private bool IsInternalProfile(string name)
        {
            if (name == TreeViewHelper.TreeSmarterNotify)
                return true;

            return false;
        }
        public void ToggleConnection(bool toggle)
        {
            string connectedProfileID = UpdateThread.CurrentProfileName;
            if (!IsReservedName(connectedProfileID))
            {
                ConnectionProfile profile = ConnectionProfileHandler.GetProfile(connectedProfileID);
                if (profile != null)
                {
                    if (toggle)
                        profile.StartConnections(false);
                    else
                        profile.StopConnections();

                    ConnectionProfileHandler.SaveProfile(profile, false, false);
                }
            }
            else
            {
                List<ConnectionProfile> profiles = new List<ConnectionProfile>();
                switch (connectedProfileID)
                {
                    case TreeViewHelper.TreeAllNotifications:
                        foreach (ConnectionProfile profile in ConnectionProfileHandler.GetAllProfiles())
                        {
                            if (toggle)
                                profile.StartConnections(false);
                            else
                                profile.StopConnections();
                        }
                        break;
                    case TreeViewHelper.TreeSmarterMail:
                        foreach (ConnectionProfile profile in ConnectionProfileHandler.GetSMProfiles())
                        {
                            if (toggle)
                                profile.StartConnections(false);
                            else
                                profile.StopConnections();
                        }
                        break;
                    case TreeViewHelper.TreeSmarterTrack:
                        foreach (ConnectionProfile profile in ConnectionProfileHandler.GetSTProfiles())
                        {
                            if (toggle)
                                profile.StartConnections(false);
                            else
                                profile.StopConnections();
                        }
                        break;
                    case TreeViewHelper.TreeSmarterStats:
                        foreach (ConnectionProfile profile in ConnectionProfileHandler.GetSSProfiles())
                        {
                            if (toggle)
                                profile.StartConnections(false);
                            else
                                profile.StopConnections();
                        }
                        break;
                    case TreeViewHelper.TreeExternalProvider:
                        foreach (ConnectionProfile profile in ConnectionProfileHandler.GetExternalProfiles())
                        {
                            if (toggle)
                                profile.StartConnections(false);
                            else
                                profile.StopConnections();
                        }
                        break;
                    case TreeViewHelper.TreePluginProvider:
                        foreach (ConnectionProfile profile in ConnectionProfileHandler.GetPluginProfiles())
                        {
                            if (toggle)
                                profile.StartConnections(false);
                            else
                                profile.StopConnections();
                        }
                        break;
                }
            }
        }
        private void ClearSelectedRowIndices()
        {
            CurrentReminderIndices.Clear();
            CurrentReminderIndices.Add(0);

            if (gvNotifications.Rows.Count == 0)
                BrowserHandler.ClearBrowser(ref webNotification);

            ShowNotificationContent();
        }
        private static void LoadConfigFiles()
        {
            AppPath.CheckLogFiles();
            SettingsHandler.LoadSettingsFromFile();
            PluginHandler.LoadPluginProviders();
            ConnectionProfileHandler.LoadProfilesFromFile();
            //ProviderHandler.LoadProvidersFromFile();
            //ReminderEventHandler.LoadEventsFromFile(); // not implemented at this time...
            InternalReminderHandler.LoadRemindersFromFile();
            VoiceDictionaryHandler.LoadVoiceEntriesFromFile();
            BrowserHandler.MakeSureFileExists();
            ConnectionProfileHandler.StartAppConnectedProfiles();
        }
        private void ToggleManageItems(bool toggle)
        {
            mnuEditConnection.Enabled = toggle;
            mnuEditConnectionDD.Enabled = toggle;
            mnuDeleteConnection.Enabled = toggle;
            mnuDeleteConnectionDD.Enabled = toggle;
        }
        public void HideForm()
        {
            WindowStateManager.CurrentState = WindowStateManager.StateTypes.Minimized;
            this.Hide();
            this.WindowState = FormWindowState.Minimized;
            if (!HasFormHidden && SettingsHandler.GlobalSettings.MessageOnFirstMinimize)
            {
                BalloonTypeAction = BalloonTypes.Minimize;
                notMain.ShowBalloonTip(500, "SmarterNotify Is Still Running", "Click here to never show this message again", ToolTipIcon.Info);
            }
            HasFormHidden = true;
        }
        public void ShowConnectedMB()
        {
            MessageBox.Show("You must be actively receiving updates from this connection profile in order to perform this action.",
                "Note", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
        }
        private void TreeViewNodeSelected(bool updateBGThread)
        {
            TreeViewNodeSelected(updateBGThread, true);
        }
        private void TreeViewNodeSelected(bool updateBGThread, bool clearSelectedRowIndices)
        {
            if (tvConnections.SelectedNode != null)
            {
                if (!IsReservedName(tvConnections.SelectedNode.Name) && !IsInternalProfile(tvConnections.SelectedNode.Name))
                    ToggleManageItems(true);
                else
                    ToggleManageItems(false);
                bool connect = true;
                if (IsInternalProfile(tvConnections.SelectedNode.Name))
                    connect = false;

                mnuConnect.Enabled = connect;
                mnuConnectDD.Enabled = connect;
                mnuDisconnect.Enabled = connect;
                mnuDisconnectDD.Enabled = connect;
                UpdateThread.CurrentProfileName = tvConnections.SelectedNode.Name;
            }
            if (updateBGThread && clearSelectedRowIndices)
                ClearSelectedRowIndices();

            UpdateWorkArea(updateBGThread);
            if (updateBGThread && WindowStateManager.CurrentState == WindowStateManager.StateTypes.Normal)
            {
                tvConnections.Focus();
                StackTrace st = new StackTrace(true);
                LogHandler.WriteLog(LogHandler.LogType.Error, "Tree view focusing " + st.ToString());
            }
        }
        private void ReadAllNotifications()
        {
            if (!VoiceThread.IsQueueProcessing_Safe())
            {
                List<Reminder> allProfileRems = ReminderHandler.GetAllProfileReminders(false);
                List<Reminder> allInternalRems = InternalReminderHandler.GetAllInternalReminders(false);

                int totalCount = (allProfileRems.Count + allInternalRems.Count);
                string notificationsMsg = totalCount > 1 || totalCount == 0 ? " notifications" : "notification";
                VoiceThread.AddToQueue("you have " + totalCount + notificationsMsg, 0, 0);

                // start reading
                int notCounter = 1;
                foreach (Reminder rem in allProfileRems)
                {
                    VoiceThread.AddToQueue("Notification number " + notCounter + ", " + rem.TimeOccurredDT, 0, 0);
                    VoiceThread.AddToQueue(string.IsNullOrEmpty(rem.PhonicSubject) ? rem.Subject : rem.PhonicSubject, 1250, 0);
                    notCounter++;
                }
                foreach (Reminder rem in allInternalRems)
                {
                    VoiceThread.AddToQueue("Notification number " + notCounter + ", " + rem.TimeOccurredDT, 0, 0);
                    VoiceThread.AddToQueue(string.IsNullOrEmpty(rem.PhonicSubject) ? rem.Subject : rem.PhonicSubject, 1250, 0);
                    notCounter++;
                }

                if (totalCount > 0)
                    VoiceThread.AddToQueue("end of notifications");
            }
            else
            {
                // stop reading
                VoiceThread.ClearQueue();
            }
        }
        #endregion

        #region Form Functionality
        private void notMain_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            if (BalloonTypeAction == BalloonTypes.Notify)
                ShowForm();
            else
            {
                SettingsHandler.GlobalSettings.MessageOnFirstMinimize = false;
                SettingsHandler.SaveSettings();
                BalloonTypeAction = BalloonTypes.Notify;
            }
        }
        private void notMain_BalloonTipClosed(object sender, EventArgs e)
        {
            BalloonTypeAction = BalloonTypes.Notify;
        }
        private void itmOpen_Click(object sender, EventArgs e)
        {
            ShowForm();
        }
        private void notMain_BalloonTipClicked(object sender, EventArgs e)
        {
            if (BalloonTypeAction == BalloonTypes.Notify)
                ShowForm();
            else
            {
                SettingsHandler.GlobalSettings.MessageOnFirstMinimize = false;
                SettingsHandler.SaveSettings();
                BalloonTypeAction = BalloonTypes.Notify;
            }
        }
        private void frmMain_HelpRequested(object sender, HelpEventArgs hlpevent)
        {
            ViewManager.OpenHelp(ViewManager.HelpCriteria.MainInterface);
        }
        private void mnuHelpTopics_Click(object sender, EventArgs e)
        {
            ViewManager.OpenHelp(ViewManager.HelpCriteria.HomePage);
        }
        private void itmHelpTopics_Click(object sender, EventArgs e)
        {
            ViewManager.OpenHelp(ViewManager.HelpCriteria.HomePage);
        }
        private void btnClose_Click(object sender, EventArgs e)
        {
            HideForm();
        }
        private void mnuCloseWindow_Click(object sender, EventArgs e)
        {
            HideForm();
        }
        private void frmMain_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (SettingsHandler.GlobalSettings.NoTaskbarMinimized)
            {
                if (e.CloseReason == CloseReason.UserClosing)
                {
                    HideForm();
                    e.Cancel = true;
                }
            }
        }
        private void frmMain_Resize(object sender, EventArgs e)
        {
            if (WindowState == FormWindowState.Minimized && SettingsHandler.GlobalSettings.NoTaskbarMinimized)
                HideForm();
        }
        private void itmExit_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }
        private void mnuExit_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }
        public void ShowDefaultSetup()
        {
            // this must be run after the treeview has been setup, usually on form load
            cboSnoozeTimeFrame.SelectedIndex = 0;
            chkShowSnoozedItems.Checked = false;
            mnuShowSnoozedItems.Checked = false;
            itmPlaySounds.Checked = SettingsHandler.GlobalSettings.NotifyPlaySound;
            ClearSelectedRowIndices();
            GenerateConnectionsArea();
            //UpdateWorkArea(true);
        }
        private void mnuAbout_Click(object sender, EventArgs e)
        {
            frmAbout about = new frmAbout();
            about.TopMost = false;
            about.ShowDialog();
        }
        private void mnuSettings_Click(object sender, EventArgs e)
        {
            frmSettings settings = new frmSettings();
            settings.OnUpdate += new frmSettings.UpdateParentList(GenerateConnectionsArea);
            settings.TopMost = false;
            settings.ShowDialog();
        }
        private void itmManageSettings_Click(object sender, EventArgs e)
        {
            ShowForm();

            frmSettings settings = new frmSettings();
            settings.OnUpdate += new frmSettings.UpdateParentList(GenerateConnectionsArea);
            settings.StartPosition = FormStartPosition.CenterScreen;
            settings.TopMost = false;
            settings.ShowDialog();
        }
        private void mnuShowSnoozedItems_Click(object sender, EventArgs e)
        {
            bool showSnoozedItems = mnuShowSnoozedItems.Checked;
            chkShowSnoozedItems.Checked = showSnoozedItems;
            UpdateThread.ShowSnoozedItems = showSnoozedItems;
            UpdateWorkArea(true);
        }
        private void chkShowSnoozedItems_CheckedChanged(object sender, EventArgs e)
        {
            bool showSnoozedItems = chkShowSnoozedItems.Checked;
            mnuShowSnoozedItems.Checked = showSnoozedItems;
            UpdateThread.ShowSnoozedItems = showSnoozedItems;
            UpdateWorkArea(true);
        }
        private void mnuShowSnoozedItems_CheckedChanged(object sender, EventArgs e)
        {
            bool showSnoozedItems = mnuShowSnoozedItems.Checked;
            chkShowSnoozedItems.Checked = showSnoozedItems;
            UpdateThread.ShowSnoozedItems = showSnoozedItems;
            UpdateWorkArea(true);
        }
        private void tvConnections_NodeMouseClick(object sender, TreeNodeMouseClickEventArgs e)
        {
            tvConnections.SelectedNode = e.Node;
            TreeViewNodeSelected(true);
        }
        private void tvConnections_NodeMouseDoubleClick(object sender, TreeNodeMouseClickEventArgs e)
        {
            tvConnections.SelectedNode = e.Node;
            TreeViewNodeSelected(true);
        }
        private void tvConnections_KeyUp(object sender, KeyEventArgs e)
        {
            TreeViewNodeSelected(true);
        }
        private void tvConnections_KeyDown(object sender, KeyEventArgs e)
        {
            TreeViewNodeSelected(true);
        }
        private void tmrAutoConnect_Tick(object sender, EventArgs e)
        {
            AutoConnectorThread.StartAutoConnectorThread();
        }
        private void tmrAsyncRequestChecker_Tick(object sender, EventArgs e)
        {
            if (!UpdateThread.AsyncThreadPerforming)
            {
                if (MultipleTVSelected)
                {
                    UpdateThread.StartAsyncUpdateThread();
                    MultipleTVSelected = false;
                }
                else
                {
                    this.Text = "SmarterNotify";
                    UpdateWorkAreaSub();
                    tmrAsyncRequestChecker.Enabled = false;
                }
            }
            else
                this.Text = "SmarterNotify (Loading)";
        }
        private void itmReadToMe_Click(object sender, EventArgs e)
        {
            ReadAllNotifications();
        }
        private void mnuReadToMe_Click(object sender, EventArgs e)
        {
            ReadAllNotifications();
        }
        private void tmrCheckVoiceThread_Tick(object sender, EventArgs e)
        {
            if (VoiceThread.IsQueueProcessing_Safe())
            {
                itmReadToMe.Text = "Stop Reading Out Loud";
                mnuReadToMe.Text = "Stop Reading Out Loud";
            }
            else
            {
                itmReadToMe.Text = "Read Out Loud Active Notifications";
                mnuReadToMe.Text = "Read Out Loud Active Notifications";
            }
        }
        private void tmrUpdater_Tick(object sender, EventArgs e)
        {
            lock (UpdateThread.NewRemindersLocker)
            {
                bool windowShowing = WindowStateManager.CurrentState != WindowStateManager.StateTypes.Minimized;
                LogHandler.WriteLog(LogHandler.LogType.Connections, "Tick Tock!");

                bool showAlert = false;
                if ((!SettingsHandler.GlobalSettings.NotifyOnlyMinimized && windowShowing) || !windowShowing)
                    showAlert = true;

                List<Reminder> reminders = UpdateThread.NewReminders;
                int numNewAlerts = reminders.Count;
                if (numNewAlerts > 0)
                    LogHandler.WriteLog(LogHandler.LogType.Notifications, "New Alerts Detected in interface",
                        "Show Alert: " + showAlert.ToString() + ", Window Showing: " + windowShowing.ToString() +
                        ", Form.WindowState: " + WindowState.ToString());
                if (showAlert && numNewAlerts > 0)
                {
                    bool showInterface = SettingsHandler.GlobalSettings.NotifyShowInterface;
                    // only repetitively flash the image if there are new alerts
                    if (!windowShowing && SettingsHandler.GlobalSettings.NotifyAnimateIcon && !showInterface)
                        tmrAlertFlasher.Enabled = true;

                    if (SettingsHandler.GlobalSettings.NotifyPopupBalloon)
                    {
                        if (numNewAlerts == 1)
                        {
                            string bodyText = reminders[0].BodyText;
                            if (bodyText.Length > 100)
                                bodyText = bodyText.Substring(0, 100) + " \r\n\r\nProfile: " + reminders[0].OwnerProfileName;

                            BalloonTypeAction = BalloonTypes.Notify;
                            notMain.ShowBalloonTip(5000, "New Alert Notifications", reminders[0].Subject + " - " + reminders[0].OwnerProfileName, ToolTipIcon.Info);
                        }
                        else
                        {
                            BalloonTypeAction = BalloonTypes.Notify;
                            notMain.ShowBalloonTip(5000, "New Alert Notifications", "Click here to open the notifications window and read your new messages", ToolTipIcon.Info);
                        }
                    }
                    if (SettingsHandler.GlobalSettings.NotifyPlaySound)
                    {
                        int soundInterval = SettingsHandler.GlobalSettings.SoundInterval;
                        bool soundAlreadyPlaying = tmrSoundInterval.Enabled;
                        if (soundInterval > -1 && !windowShowing && !showInterface)
                        {
                            tmrSoundInterval.Interval = soundInterval;
                            tmrSoundInterval.Enabled = true;
                        }
                        if (!soundAlreadyPlaying)
                            PlaySound();
                    }
                    if (SettingsHandler.GlobalSettings.NotifyReadToMe)
                    {
                        string messageToQueue = !string.IsNullOrEmpty(reminders[0].PhonicSubject) ? reminders[0].PhonicSubject : reminders[0].Subject;
                        VoiceThread.AddToQueue(numNewAlerts > 1 ? "You Have New Alert Notifications" : messageToQueue);
                    }
                    if (!windowShowing && showInterface)
                    {
                        this.Show();
                        StackTrace st = new StackTrace(true);
                        LogHandler.WriteLog(LogHandler.LogType.Error, "Window showing " + st.ToString());

                        WindowState = FormWindowState.Normal;
                        WindowStateManager.CurrentState = WindowStateManager.StateTypes.Normal;
                    }
                    else if (showInterface)
                        this.Activate();
                }

                bool isNewAlerts = numNewAlerts > 0;
                if (isNewAlerts)
                {
                    Reminder lastReminder = null;
                    foreach (Reminder reminder in reminders)
                    {
                        TrackingHelper.TrackNotification(reminder.Subject, reminder.BodyText);
                        lastReminder = reminder;
                    }
                    if (lastReminder != null && SettingsHandler.GlobalSettings.AutoNavigateNewAlerts)
                    {
                        // select the last reminder's profile on the left
                        ConnectionProfile profile = ConnectionProfileHandler.GetProfile(lastReminder.ProfileGUID);
                        if (profile != null)
                        {
                            string nodeName = string.Empty;
                            switch (profile.ConnectionType)
                            {
                                case ConnectionProfile.ConnectionProfileType.SmarterTrack: nodeName = TreeViewHelper.TreeSmarterTrack; break;
                                case ConnectionProfile.ConnectionProfileType.SmarterMail: nodeName = TreeViewHelper.TreeSmarterMail; break;
                                case ConnectionProfile.ConnectionProfileType.SmarterStats: nodeName = TreeViewHelper.TreeSmarterStats; break;
                                case ConnectionProfile.ConnectionProfileType.ExternalProvider: nodeName = TreeViewHelper.TreeExternalProvider; break;
                                case ConnectionProfile.ConnectionProfileType.PluginProvider: nodeName = TreeViewHelper.TreePluginProvider; break;
                            }
                            foreach (TreeNode node in tvConnections.Nodes)
                            {
                                if (node.Name == nodeName)
                                {
                                    bool childSelected = false;
                                    foreach (TreeNode childNode in node.Nodes)
                                    {
                                        if (childNode.Name == profile.ProfileGUID)
                                        {
                                            tvConnections.SelectedNode = childNode;
                                            childSelected = true;
                                            break;
                                        }
                                    }
                                    if (!childSelected)
                                        tvConnections.SelectedNode = node;
                                    break;
                                }
                            }
                        }
                    }

                    ViewManager.UpdateLastCheckDate();
                    UpdateThread.NewReminders.Clear();
                    LogHandler.WriteLog(LogHandler.LogType.Notifications, "New Alerts Cleared", "All the new alerts and their counts have been cleared");
                }

                TreeViewNodeSelected(isNewAlerts, SettingsHandler.GlobalSettings.AutoNavigateNewAlerts);
            }
        }
        private void gvNotifications_MouseClick(object sender, MouseEventArgs e)
        {
            SelectNotification();
        }
        private void gvNotifications_KeyUp(object sender, KeyEventArgs e)
        {
            SelectNotification();
        }
        private void gvNotifications_KeyDown(object sender, KeyEventArgs e)
        {
            SelectNotification();
        }
        private void SelectNotification()
        {
            lock (RemindersKey)
            {
                foreach (DataGridViewRow row in gvNotifications.Rows)
                {
                    if (row.Selected && row.Index > -1 && !CurrentReminderIndices.Contains(row.Index))
                        CurrentReminderIndices.Add(row.Index);
                    else if (!row.Selected && row.Index > -1 && CurrentReminderIndices.Contains(row.Index))
                        CurrentReminderIndices.Remove(row.Index);
                }
                ShowNotificationContent();
            }
        }
        private void SelectCurrentIndices()
        {
            lock (RemindersKey)
            {
                int maxVal = -1;
                int index = CurrentReminderIndices.Count - 1;
                if (index > -1)
                    maxVal = CurrentReminderIndices[index];

                // select the cell to show the black arrow
                if (gvNotifications.Rows.Count > maxVal)
                {
                    if (maxVal > 0 && gvNotifications.Rows[maxVal].Cells.Count >= 4)
                    {
                        foreach (DataGridViewCell cell in gvNotifications.Rows[maxVal].Cells)
                            if (cell.Visible)
                            {
                                gvNotifications.CurrentCell = cell;
                                break;
                            }
                    }
                }

                foreach (int currentIndex in CurrentReminderIndices)
                {
                    if (gvNotifications.Rows.Count > currentIndex)
                    {
                        gvNotifications.Rows[currentIndex].Selected = true;
                        if (currentIndex > maxVal)
                            maxVal = currentIndex;
                    }
                }
            }
        }
        private void tmrAlertFlasher_Tick(object sender, EventArgs e)
        {
            try
            {
                if (NextTaskbarIcon == TaskbarIconTypes.Normal)
                {
                    notMain.Icon = new Icon("Images\\Yellow-white-Notify.ico");
                    NextTaskbarIcon = TaskbarIconTypes.Alert;
                }
                else
                {
                    notMain.Icon = new Icon("Images\\Yellow_Notify.ico");
                    NextTaskbarIcon = TaskbarIconTypes.Normal;
                }
            }
            catch (Exception ex)
            {
                LogHandler.WriteLog(LogHandler.LogType.Error, "There was an error that occurred while attempting to flash the task tray icon", ex.Message, ex.StackTrace);
            }
        }
        private void tmrSoundInterval_Tick(object sender, EventArgs e)
        {
            PlaySound();
        }
        private void PlaySound()
        {
            try
            {
                System.Media.SoundPlayer player = new System.Media.SoundPlayer();

                string usedFile = SettingsHandler.GlobalSettings.GetSoundFile();
                bool canPlaySound = false;
                if (!File.Exists(usedFile))
                {
                    if (File.Exists("Sounds\\NewReminder.wav"))
                    {
                        canPlaySound = true;
                        usedFile = "Sounds\\NewReminder.wav";
                    }
                }
                else
                    canPlaySound = true;

                if (canPlaySound)
                {
                    player.SoundLocation = usedFile;

                    player.Load();
                    player.Play();
                }
            }
            catch (Exception ex)
            {
                string logMsg = "Error occurred while loading/playing the notification sound";
                TrackingHelper.TrackError(TrackingHelper.RecordType.Both, "Error Occurred", logMsg, logMsg, ex);
            }
        }
        private void ShowForm()
        {
            WindowStateManager.CurrentState = WindowStateManager.StateTypes.Normal;
            Show();
            StackTrace st = new StackTrace(true);
            LogHandler.WriteLog(LogHandler.LogType.Error, "Window show " + st.ToString());
            WindowState = FormWindowState.Normal;
            ViewManager.UpdateLastCheckDate();
            ClearNoteStatus();            
        }
        private void ClearNoteStatus()
        {
            tmrAlertFlasher.Enabled = false;
            tmrSoundInterval.Enabled = false;
            notMain.Icon = new Icon("Images\\Blue_Notify.ico");
            NextTaskbarIcon = TaskbarIconTypes.Normal;
            LogHandler.WriteLog(LogHandler.LogType.Notifications, "Notification icon status", "The notification status icon has returned to normal. It has hit the ClearNoteStatus() method.");
        }
        #endregion
    }// class
}// namespace