﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using SmarterNotify.Data;
using SmarterNotify.DataHandlers;
using System.IO;
using SmarterNotify.HelperClasses;
using SmarterNotify.Threads;

namespace SmarterNotify
{
    public partial class frmSettings : Form
    {
        public frmSettings()
        {
            InitializeComponent();
        }
        public enum LoggingTypes
        {
            Disabled = 0,
            EnableLogging = 1,
            EnableNotifications = 2,
            EnableLoggingNotifications = 3
        }
        public delegate void UpdateParentList();
        public event UpdateParentList OnUpdate;

        private void frmSettings_Load(object sender, EventArgs e)
        {
            txtNotificationSound.Text = SettingsHandler.GlobalSettings.GetSoundFile();
            chkReadToMe.Checked = SettingsHandler.GlobalSettings.NotifyReadToMe;
            chkBalloons.Checked = SettingsHandler.GlobalSettings.NotifyPopupBalloon;
            chkAnimate.Checked = SettingsHandler.GlobalSettings.NotifyAnimateIcon;
            chkPlaySound.Checked = SettingsHandler.GlobalSettings.NotifyPlaySound;
            chkShowSmarterNotify.Checked = SettingsHandler.GlobalSettings.NotifyShowInterface;
            chkNoTaskbar.Checked = SettingsHandler.GlobalSettings.NoTaskbarMinimized;
            chkNotifyMinimized.Checked = SettingsHandler.GlobalSettings.NotifyOnlyMinimized;
            chkAutoNavigate.Checked = SettingsHandler.GlobalSettings.AutoNavigateNewAlerts;
            chkMinimizeOnDismissAll.Checked = SettingsHandler.GlobalSettings.MinimizeOnDismissAll;

            bool errorLogging = SettingsHandler.GlobalSettings.EnableErrorLogging;
            bool errorNotify = SettingsHandler.GlobalSettings.EnableErrorNotifications;
            bool connectionLogging = SettingsHandler.GlobalSettings.EnableConnectionLogging;
            bool connectionNotify = SettingsHandler.GlobalSettings.EnableConnectionNotifications;
            bool notifyLogging = SettingsHandler.GlobalSettings.EnableNotificationLogging;

            cboErrorLogging.Items.Clear();
            cboErrorLogging.Items.Add("Disabled");
            cboErrorLogging.Items.Add("Enable Logging");
            cboErrorLogging.Items.Add("Enable Notifications");
            cboErrorLogging.Items.Add("Enable Logging & Notifications");
            if (!errorLogging && !errorNotify)
                cboErrorLogging.SelectedIndex = 0;
            else if (errorLogging && !errorNotify)
                cboErrorLogging.SelectedIndex = 1;
            else if (!errorLogging && errorNotify)
                cboErrorLogging.SelectedIndex = 2;
            else if (errorLogging && errorNotify)
                cboErrorLogging.SelectedIndex = 3;

            cboConnectionLogging.Items.Clear();
            cboConnectionLogging.Items.Add("Disabled");
            cboConnectionLogging.Items.Add("Enable Logging");
            cboConnectionLogging.Items.Add("Enable Notifications");
            cboConnectionLogging.Items.Add("Enable Logging & Notifications");
            if (!connectionLogging && !connectionNotify)
                cboConnectionLogging.SelectedIndex = 0;
            else if (connectionLogging && !connectionNotify)
                cboConnectionLogging.SelectedIndex = 1;
            else if (!connectionLogging && connectionNotify)
                cboConnectionLogging.SelectedIndex = 2;
            else if (connectionLogging && connectionNotify)
                cboConnectionLogging.SelectedIndex = 3;

            cboNotificationLogging.Items.Clear();
            cboNotificationLogging.Items.Add("Disabled");
            cboNotificationLogging.Items.Add("Enable Logging");
            cboNotificationLogging.SelectedIndex = notifyLogging ? 1 : 0;

            List<TabPage> tabsToRemove = new List<TabPage>();
            foreach (TabPage tab in tabSettings.TabPages)
                if (tab.Name == "tabErrorConditions" || tab.Name == "tabExternalProviders")
                    tabsToRemove.Add(tab);
            
            foreach (TabPage tab in tabsToRemove)
                tabSettings.TabPages.Remove(tab);

            if (cboSoundInterval.Items.Count >= 7)
            {
                switch (SettingsHandler.GlobalSettings.SoundInterval)
                {
                    case -1: cboSoundInterval.SelectedIndex = 0; break;
                    case 15000: cboSoundInterval.SelectedIndex = 1; break;
                    case 30000: cboSoundInterval.SelectedIndex = 2; break;
                    case 60000: cboSoundInterval.SelectedIndex = 3; break;
                    case 120000: cboSoundInterval.SelectedIndex = 4; break;
                    case 300000: cboSoundInterval.SelectedIndex = 5; break;
                    case 600000: cboSoundInterval.SelectedIndex = 6; break;
                }
            }
        }
        private void btnSave_Click(object sender, EventArgs e)
        {
            try
            {
                // save general settings
                Settings newSettings = new Settings();

                FileInfo info = new FileInfo(txtNotificationSound.Text);
                if (info != null && info.Extension.ToUpperInvariant() == ".WAV" && File.Exists(txtNotificationSound.Text))
                    newSettings.SetSoundFile(txtNotificationSound.Text);
                else
                {
                    ShowErrorMessage("The notification sound file you selected is invalid.");
                    return;
                }

                newSettings.NotifyPopupBalloon = chkBalloons.Checked;
                newSettings.NotifyAnimateIcon = chkAnimate.Checked;
                newSettings.NotifyPlaySound = chkPlaySound.Checked;
                newSettings.NotifyShowInterface = chkShowSmarterNotify.Checked;
                newSettings.NoTaskbarMinimized = chkNoTaskbar.Checked;
                newSettings.NotifyReadToMe = chkReadToMe.Checked;
                newSettings.NotifyOnlyMinimized = chkNotifyMinimized.Checked;
                
                newSettings.EnableErrorLogging = cboErrorLogging.SelectedItem.ToString().ToUpperInvariant().Contains("LOGGING");
                newSettings.EnableErrorNotifications = cboErrorLogging.SelectedItem.ToString().ToUpperInvariant().Contains("NOTIFICATIONS");
                
                newSettings.EnableConnectionLogging = cboConnectionLogging.SelectedItem.ToString().ToUpperInvariant().Contains("LOGGING");
                newSettings.EnableConnectionNotifications = cboConnectionLogging.SelectedItem.ToString().ToUpperInvariant().Contains("NOTIFICATIONS");

                newSettings.EnableNotificationLogging = cboNotificationLogging.SelectedItem.ToString().ToUpperInvariant().Contains("LOGGING");
                newSettings.AutoNavigateNewAlerts = chkAutoNavigate.Checked;
                newSettings.MinimizeOnDismissAll = chkMinimizeOnDismissAll.Checked;
                newSettings.MessageOnFirstMinimize = SettingsHandler.GlobalSettings.MessageOnFirstMinimize;
                newSettings.IsFirstLoad = false;

                switch (cboSoundInterval.SelectedItem.ToString().ToUpperInvariant())
                {
                    case "ONLY PLAY ONCE": newSettings.SoundInterval = -1; break;
                    case "15 SECONDS": newSettings.SoundInterval = 15000; break;
                    case "30 SECONDS": newSettings.SoundInterval = 30000; break;
                    case "1 MINUTE": newSettings.SoundInterval = 60000; break;
                    case "2 MINUTES": newSettings.SoundInterval = 120000; break;
                    case "5 MINUTES": newSettings.SoundInterval = 300000; break;
                    case "10 MINUTES": newSettings.SoundInterval = 600000; break;
                    default: newSettings.SoundInterval = -1; break;
                }

                SettingsHandler.SaveSettings(newSettings);

                if (SettingsHandler.GlobalSettings.ShouldShowSmarterNotify())
                    UpdateThread.CurrentProfileName = TreeViewHelper.TreeSmarterNotify;
                else
                    UpdateThread.CurrentProfileName = TreeViewHelper.TreeAllNotifications;

                OnUpdate();
                this.Close();
            }
            catch (Exception ex) 
            {
                string logMsg = "Error occurred while saving settings";
                TrackingHelper.TrackError(TrackingHelper.RecordType.Both, "Error Occurred", logMsg, logMsg, ex);
            }
        }
        private void btnCancel_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void btnBrowseSound_Click(object sender, EventArgs e)
        {
            DialogResult result = opnFile.ShowDialog();
            if (result == DialogResult.OK || result == DialogResult.Yes)
                txtNotificationSound.Text = opnFile.FileName;
        }

        private void lnkViewErrorLog_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            try
            {
                System.Diagnostics.Process.Start(AppPath.ErrorLogFilePath);
            }
            catch { MessageBox.Show("Please try again in a few moments"); }
        }

        private void lnkViewConnectionLog_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            try
            {
                System.Diagnostics.Process.Start(AppPath.ConnectionLogFilePath);
            }
            catch { MessageBox.Show("Please try again in a few moments"); }
        }

        private void lnkViewNotificationLog_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            try
            {
                System.Diagnostics.Process.Start(AppPath.NotificationLogFilePath);
            }
            catch { MessageBox.Show("Please try again in a few moments"); }
        }

        private void lnkClearErrorLog_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            try
            {
                LogHandler.ClearLog(LogHandler.LogType.Error);
                MessageBox.Show("Your error log has been cleared", "Message", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            catch { MessageBox.Show("Please try again in a few moments"); }
        }

        private void lnkClearConnectionLog_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            try
            {
                LogHandler.ClearLog(LogHandler.LogType.Connections);
                MessageBox.Show("Your connection log has been cleared", "Message", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            catch { MessageBox.Show("Please try again in a few moments"); }
        }

        private void lnkClearNotificationLog_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            try
            {
                LogHandler.ClearLog(LogHandler.LogType.Notifications);
                MessageBox.Show("Your notification log has been cleared", "Message", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            catch { MessageBox.Show("Please try again in a few moments"); }
        }

        private void btnNewCondition_Click(object sender, EventArgs e)
        {
            //frmNotificationEvent notEvent = new frmNotificationEvent();
            //notEvent.ShowDialog();
        }

        private void btnEditCondition_Click(object sender, EventArgs e)
        {
            //frmNotificationEvent notEvent = new frmNotificationEvent();
            //notEvent.ShowDialog();
        }

        private void btnDeleteCondition_Click(object sender, EventArgs e)
        {

        }

        #region Helper Methods

        private void ShowErrorMessage(string message)
        {
            MessageBox.Show(message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }

        #endregion

        private void btnNewProvider_Click(object sender, EventArgs e)
        {
            //frmProvider provider = new frmProvider();
            //provider.OnUpdate += new frmProvider.OnUpdateDel(provider_OnUpdate);
            //provider.ShowDialog();
        }
        private void btnEditProvider_Click(object sender, EventArgs e)
        {
            //frmProvider provider = new frmProvider();
            //provider.OnUpdate += new frmProvider.OnUpdateDel(provider_OnUpdate);
        }
        private void btnDeleteProvider_Click(object sender, EventArgs e)
        {

        }
        public void provider_OnUpdate()
        {
            throw new NotImplementedException();
        }

        private void chkPlaySound_CheckedChanged(object sender, EventArgs e)
        {
            ToggleSoundOptionVisibility(chkPlaySound.Checked);
        }
        public void ToggleSoundOptionVisibility(bool toggle)
        {
            txtNotificationSound.Visible = toggle;
            btnBrowseSound.Visible = toggle;
            lblNote1.Visible = toggle;
            lblNotificationSound.Visible = toggle;
            cboSoundInterval.Visible = toggle;
            lblNote2.Visible = toggle;
            lblSoundInterval.Visible = toggle;
        }
        private void btnHelp_Click(object sender, EventArgs e)
        {
            ViewManager.OpenHelp(ViewManager.HelpCriteria.Settings);
        }
        private void frmSettings_HelpRequested(object sender, HelpEventArgs hlpevent)
        {
            ViewManager.OpenHelp(ViewManager.HelpCriteria.Settings);
        }

        private void btnBrowseSound_Click_1(object sender, EventArgs e)
        {
            DialogResult result = opnFile.ShowDialog();
            if (result == DialogResult.OK || result == DialogResult.Yes)
                txtNotificationSound.Text = opnFile.FileName;
        }
    }// class
}// namespace